from typing import List

from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from ....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Unused:
	"""Unused commands group definition. 8 total commands, 1 Subgroups, 6 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("unused", core, parent)

	@property
	def path(self):
		"""path commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_path'):
			from .Path import Path
			self._path = Path(self._core, self._cmd_group)
		return self._path

	def get_all(self) -> bool:
		"""SCPI: CPANel:UNUSed:ALL \n
		Snippet: value: bool = driver.cpanel.unused.get_all() \n
		Shows all unused signal generators. \n
			:return: all_py: ON| OFF| 1| 0
		"""
		response = self._core.io.query_str('CPANel:UNUSed:ALL?')
		return Conversions.str_to_bool(response)

	def set_all(self, all_py: bool) -> None:
		"""SCPI: CPANel:UNUSed:ALL \n
		Snippet: driver.cpanel.unused.set_all(all_py = False) \n
		Shows all unused signal generators. \n
			:param all_py: ON| OFF| 1| 0
		"""
		param = Conversions.bool_to_str(all_py)
		self._core.io.write(f'CPANel:UNUSed:ALL {param}')

	def get_frequency(self) -> float:
		"""SCPI: CPANel:UNUSed:FREQuency \n
		Snippet: value: float = driver.cpanel.unused.get_frequency() \n
		Sets the RF frequency. \n
			:return: frequency: float
		"""
		response = self._core.io.query_str('CPANel:UNUSed:FREQuency?')
		return Conversions.str_to_float(response)

	def set_frequency(self, frequency: float) -> None:
		"""SCPI: CPANel:UNUSed:FREQuency \n
		Snippet: driver.cpanel.unused.set_frequency(frequency = 1.0) \n
		Sets the RF frequency. \n
			:param frequency: float
		"""
		param = Conversions.decimal_value_to_str(frequency)
		self._core.io.write(f'CPANel:UNUSed:FREQuency {param}')

	def get_level(self) -> float:
		"""SCPI: CPANel:UNUSed:LEVel \n
		Snippet: value: float = driver.cpanel.unused.get_level() \n
		Sets the RF level. \n
			:return: level: float
		"""
		response = self._core.io.query_str('CPANel:UNUSed:LEVel?')
		return Conversions.str_to_float(response)

	def set_level(self, level: float) -> None:
		"""SCPI: CPANel:UNUSed:LEVel \n
		Snippet: driver.cpanel.unused.set_level(level = 1.0) \n
		Sets the RF level. \n
			:param level: float
		"""
		param = Conversions.decimal_value_to_str(level)
		self._core.io.write(f'CPANel:UNUSed:LEVel {param}')

	def get_list_py(self) -> List[int]:
		"""SCPI: CPANel:UNUSed:LIST \n
		Snippet: value: List[int] = driver.cpanel.unused.get_list_py() \n
		Queries the names of the used/unused signal generators. \n
			:return: list_py: 'Instr#1','Instr#2',...
		"""
		response = self._core.io.query_bin_or_ascii_int_list('CPANel:UNUSed:LIST?')
		return response

	def get_select(self) -> str:
		"""SCPI: CPANel:UNUSed:SELect \n
		Snippet: value: str = driver.cpanel.unused.get_select() \n
		Selects the element to which the subsequent commands apply. \n
			:return: select: string Available element as queried with the corresponding ...:LIST command. For example, method RsPulseSeq.Assignment.Generator.Path.Antenna.listPy
		"""
		response = self._core.io.query_str('CPANel:UNUSed:SELect?')
		return trim_str_response(response)

	def set_select(self, select: str) -> None:
		"""SCPI: CPANel:UNUSed:SELect \n
		Snippet: driver.cpanel.unused.set_select(select = '1') \n
		Selects the element to which the subsequent commands apply. \n
			:param select: string Available element as queried with the corresponding ...:LIST command. For example, method RsPulseSeq.Assignment.Generator.Path.Antenna.listPy
		"""
		param = Conversions.value_to_quoted_str(select)
		self._core.io.write(f'CPANel:UNUSed:SELect {param}')

	def get_state(self) -> bool:
		"""SCPI: CPANel:UNUSed:STATe \n
		Snippet: value: bool = driver.cpanel.unused.get_state() \n
		Activates the RF output of the selected signal generator. \n
			:return: state: ON| OFF| 1| 0
		"""
		response = self._core.io.query_str('CPANel:UNUSed:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, state: bool) -> None:
		"""SCPI: CPANel:UNUSed:STATe \n
		Snippet: driver.cpanel.unused.set_state(state = False) \n
		Activates the RF output of the selected signal generator. \n
			:param state: ON| OFF| 1| 0
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'CPANel:UNUSed:STATe {param}')

	def clone(self) -> 'Unused':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = Unused(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from ....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Antenna:
	"""Antenna commands group definition. 17 total commands, 3 Subgroups, 8 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("antenna", core, parent)

	@property
	def add(self):
		"""add commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_add'):
			from .Add import Add
			self._add = Add(self._core, self._cmd_group)
		return self._add

	@property
	def direction(self):
		"""direction commands group. 0 Sub-classes, 3 commands."""
		if not hasattr(self, '_direction'):
			from .Direction import Direction
			self._direction = Direction(self._core, self._cmd_group)
		return self._direction

	@property
	def position(self):
		"""position commands group. 0 Sub-classes, 5 commands."""
		if not hasattr(self, '_position'):
			from .Position import Position
			self._position = Position(self._core, self._cmd_group)
		return self._position

	def get_alias(self) -> str:
		"""SCPI: RECeiver:ANTenna:ALIas \n
		Snippet: value: str = driver.receiver.antenna.get_alias() \n
		Sets an alias name for the selected antenna element. \n
			:return: alias: string
		"""
		response = self._core.io.query_str('RECeiver:ANTenna:ALIas?')
		return trim_str_response(response)

	def set_alias(self, alias: str) -> None:
		"""SCPI: RECeiver:ANTenna:ALIas \n
		Snippet: driver.receiver.antenna.set_alias(alias = '1') \n
		Sets an alias name for the selected antenna element. \n
			:param alias: string
		"""
		param = Conversions.value_to_quoted_str(alias)
		self._core.io.write(f'RECeiver:ANTenna:ALIas {param}')

	def get_bm(self) -> str:
		"""SCPI: RECeiver:ANTenna:BM \n
		Snippet: value: str = driver.receiver.antenna.get_bm() \n
		No command help available \n
			:return: bm: No help available
		"""
		response = self._core.io.query_str('RECeiver:ANTenna:BM?')
		return trim_str_response(response)

	def set_bm(self, bm: str) -> None:
		"""SCPI: RECeiver:ANTenna:BM \n
		Snippet: driver.receiver.antenna.set_bm(bm = '1') \n
		No command help available \n
			:param bm: No help available
		"""
		param = Conversions.value_to_quoted_str(bm)
		self._core.io.write(f'RECeiver:ANTenna:BM {param}')

	def clear(self) -> None:
		"""SCPI: RECeiver:ANTenna:CLEar \n
		Snippet: driver.receiver.antenna.clear() \n
		Deletes all items from the list or the table. \n
		"""
		self._core.io.write(f'RECeiver:ANTenna:CLEar')

	def clear_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: RECeiver:ANTenna:CLEar \n
		Snippet: driver.receiver.antenna.clear_with_opc() \n
		Deletes all items from the list or the table. \n
		Same as clear, but waits for the operation to complete before continuing further. Use the RsPulseSeq.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'RECeiver:ANTenna:CLEar', opc_timeout_ms)

	def delete(self) -> None:
		"""SCPI: RECeiver:ANTenna:DELete \n
		Snippet: driver.receiver.antenna.delete() \n
		Deletes the particular item. \n
		"""
		self._core.io.write(f'RECeiver:ANTenna:DELete')

	def delete_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: RECeiver:ANTenna:DELete \n
		Snippet: driver.receiver.antenna.delete_with_opc() \n
		Deletes the particular item. \n
		Same as delete, but waits for the operation to complete before continuing further. Use the RsPulseSeq.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'RECeiver:ANTenna:DELete', opc_timeout_ms)

	def get_gain(self) -> float:
		"""SCPI: RECeiver:ANTenna:GAIN \n
		Snippet: value: float = driver.receiver.antenna.get_gain() \n
		Sets the gain of the individual antenna element. \n
			:return: gain: float Range: -120 to 120
		"""
		response = self._core.io.query_str('RECeiver:ANTenna:GAIN?')
		return Conversions.str_to_float(response)

	def set_gain(self, gain: float) -> None:
		"""SCPI: RECeiver:ANTenna:GAIN \n
		Snippet: driver.receiver.antenna.set_gain(gain = 1.0) \n
		Sets the gain of the individual antenna element. \n
			:param gain: float Range: -120 to 120
		"""
		param = Conversions.decimal_value_to_str(gain)
		self._core.io.write(f'RECeiver:ANTenna:GAIN {param}')

	def get_pattern(self) -> str:
		"""SCPI: RECeiver:ANTenna:PATTern \n
		Snippet: value: str = driver.receiver.antenna.get_pattern() \n
		Assigns an existing antenna pattern, see method RsPulseSeq.Antenna.catalog. \n
			:return: pattern: string
		"""
		response = self._core.io.query_str('RECeiver:ANTenna:PATTern?')
		return trim_str_response(response)

	def set_pattern(self, pattern: str) -> None:
		"""SCPI: RECeiver:ANTenna:PATTern \n
		Snippet: driver.receiver.antenna.set_pattern(pattern = '1') \n
		Assigns an existing antenna pattern, see method RsPulseSeq.Antenna.catalog. \n
			:param pattern: string
		"""
		param = Conversions.value_to_quoted_str(pattern)
		self._core.io.write(f'RECeiver:ANTenna:PATTern {param}')

	def get_scan(self) -> str:
		"""SCPI: RECeiver:ANTenna:SCAN \n
		Snippet: value: str = driver.receiver.antenna.get_scan() \n
		Sets the antenna scan. \n
			:return: scan: string
		"""
		response = self._core.io.query_str('RECeiver:ANTenna:SCAN?')
		return trim_str_response(response)

	def set_scan(self, scan: str) -> None:
		"""SCPI: RECeiver:ANTenna:SCAN \n
		Snippet: driver.receiver.antenna.set_scan(scan = '1') \n
		Sets the antenna scan. \n
			:param scan: string
		"""
		param = Conversions.value_to_quoted_str(scan)
		self._core.io.write(f'RECeiver:ANTenna:SCAN {param}')

	def get_select(self) -> float:
		"""SCPI: RECeiver:ANTenna:SELect \n
		Snippet: value: float = driver.receiver.antenna.get_select() \n
		Selects the item to which the subsequent commands apply. \n
			:return: select: float Item number within the range 1 to ...:COUNt. For example, method RsPulseSeq.Sequence.Item.count. Range: 1 to 4096
		"""
		response = self._core.io.query_str('RECeiver:ANTenna:SELect?')
		return Conversions.str_to_float(response)

	def set_select(self, select: float) -> None:
		"""SCPI: RECeiver:ANTenna:SELect \n
		Snippet: driver.receiver.antenna.set_select(select = 1.0) \n
		Selects the item to which the subsequent commands apply. \n
			:param select: float Item number within the range 1 to ...:COUNt. For example, method RsPulseSeq.Sequence.Item.count. Range: 1 to 4096
		"""
		param = Conversions.decimal_value_to_str(select)
		self._core.io.write(f'RECeiver:ANTenna:SELect {param}')

	def clone(self) -> 'Antenna':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = Antenna(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from .... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Am:
	"""Am commands group definition. 3 total commands, 0 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("am", core, parent)

	def get_frequency(self) -> float:
		"""SCPI: PULSe:MOP:AM:FREQuency \n
		Snippet: value: float = driver.pulse.mop.am.get_frequency() \n
		Sets modulation frequency. \n
			:return: frequency: float Range: 0.001 to 1e+09
		"""
		response = self._core.io.query_str('PULSe:MOP:AM:FREQuency?')
		return Conversions.str_to_float(response)

	def set_frequency(self, frequency: float) -> None:
		"""SCPI: PULSe:MOP:AM:FREQuency \n
		Snippet: driver.pulse.mop.am.set_frequency(frequency = 1.0) \n
		Sets modulation frequency. \n
			:param frequency: float Range: 0.001 to 1e+09
		"""
		param = Conversions.decimal_value_to_str(frequency)
		self._core.io.write(f'PULSe:MOP:AM:FREQuency {param}')

	def get_mdepth(self) -> float:
		"""SCPI: PULSe:MOP:AM:MDEPth \n
		Snippet: value: float = driver.pulse.mop.am.get_mdepth() \n
		Sets the modulation depth. \n
			:return: mdepth: float Range: 0 to 100, Unit: percent
		"""
		response = self._core.io.query_str('PULSe:MOP:AM:MDEPth?')
		return Conversions.str_to_float(response)

	def set_mdepth(self, mdepth: float) -> None:
		"""SCPI: PULSe:MOP:AM:MDEPth \n
		Snippet: driver.pulse.mop.am.set_mdepth(mdepth = 1.0) \n
		Sets the modulation depth. \n
			:param mdepth: float Range: 0 to 100, Unit: percent
		"""
		param = Conversions.decimal_value_to_str(mdepth)
		self._core.io.write(f'PULSe:MOP:AM:MDEPth {param}')

	# noinspection PyTypeChecker
	def get_type_py(self) -> enums.AmType:
		"""SCPI: PULSe:MOP:AM:TYPE \n
		Snippet: value: enums.AmType = driver.pulse.mop.am.get_type_py() \n
		Selects the modulation type. \n
			:return: type_py: STD| LSB| USB| SB
		"""
		response = self._core.io.query_str('PULSe:MOP:AM:TYPE?')
		return Conversions.str_to_scalar_enum(response, enums.AmType)

	def set_type_py(self, type_py: enums.AmType) -> None:
		"""SCPI: PULSe:MOP:AM:TYPE \n
		Snippet: driver.pulse.mop.am.set_type_py(type_py = enums.AmType.LSB) \n
		Selects the modulation type. \n
			:param type_py: STD| LSB| USB| SB
		"""
		param = Conversions.enum_scalar_to_str(type_py, enums.AmType)
		self._core.io.write(f'PULSe:MOP:AM:TYPE {param}')



from typing import ClassVar, List

from .Internal.Core import Core
from .Internal.InstrumentErrors import RsInstrException
from .Internal.CommandsGroup import CommandsGroup
from .Internal.VisaSession import VisaSession
from datetime import datetime, timedelta


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class RsPulseSeq:
	"""1229 total commands, 29 Subgroups, 0 group commands"""
	_driver_options = "SupportedInstrModels = PulseSequencer, SupportedIdnPatterns = Pulse\s*Sequencer, SkipStatusSystemSettings = 1, SimulationIdnString = 'Rohde&Schwarz,PulseSequencer,100001,2.4.0.0021'"
	_global_logging_relative_timestamp: ClassVar[datetime] = None
	_global_logging_target_stream: ClassVar = None

	def __init__(self, resource_name: str, id_query: bool = True, reset: bool = False, options: str = None, direct_session: object = None):
		"""Initializes new RsPulseSeq session. \n
		Parameter options tokens examples:
			- ``Simulate=True`` - starts the session in simulation mode. Default: ``False``
			- ``SelectVisa=socket`` - uses no VISA implementation for socket connections - you do not need any VISA-C installation
			- ``SelectVisa=rs`` - forces usage of RohdeSchwarz Visa
			- ``SelectVisa=ivi`` - forces usage of National Instruments Visa
			- ``QueryInstrumentStatus = False`` - same as ``driver.utilities.instrument_status_checking = False``. Default: ``True``
			- ``WriteDelay = 20, ReadDelay = 5`` - Introduces delay of 20ms before each write and 5ms before each read. Default: ``0ms`` for both
			- ``OpcWaitMode = OpcQuery`` - mode for all the opc-synchronised write/reads. Other modes: StbPolling, StbPollingSlow, StbPollingSuperSlow. Default: ``StbPolling``
			- ``AddTermCharToWriteBinBLock = True`` - Adds one additional LF to the end of the binary data (some instruments require that). Default: ``False``
			- ``AssureWriteWithTermChar = True`` - Makes sure each command/query is terminated with termination character. Default: Interface dependent
			- ``TerminationCharacter = "\\r"`` - Sets the termination character for reading. Default: ``\\n`` (LineFeed or LF)
			- ``DataChunkSize = 10E3`` - Maximum size of one write/read segment. If transferred data is bigger, it is split to more segments. Default: ``1E6`` bytes
			- ``OpcTimeout = 10000`` - same as driver.utilities.opc_timeout = 10000. Default: ``30000ms``
			- ``VisaTimeout = 5000`` - same as driver.utilities.visa_timeout = 5000. Default: ``10000ms``
			- ``ViClearExeMode = Disabled`` - viClear() execution mode. Default: ``execute_on_all``
			- ``OpcQueryAfterWrite = True`` - same as driver.utilities.opc_query_after_write = True. Default: ``False``
			- ``StbInErrorCheck = False`` - if true, the driver checks errors with *STB? If false, it uses SYST:ERR?. Default: ``True``
			- ``LoggingMode = On`` - Sets the logging status right from the start. Default: ``Off``
			- ``LoggingName = 'MyDevice'`` - Sets the name to represent the session in the log entries. Default: ``'resource_name'``
			- ``LogToGlobalTarget = True`` - Sets the logging target to the class-property previously set with RsPulseSeq.set_global_logging_target() Default: ``False``
			- ``LoggingToConsole = True`` - Immediately starts logging to the console. Default: False
			- ``LoggingToUdp = True`` - Immediately starts logging to the UDP port. Default: False
			- ``LoggingUdpPort = 49200`` - UDP port to log to. Default: 49200
		:param resource_name: VISA resource name, e.g. 'TCPIP::192.168.2.1::INSTR'
		:param id_query: if True, the instrument's model name is verified against the models supported by the driver and eventually throws an exception.
		:param reset: Resets the instrument (sends *RST command) and clears its status sybsystem.
		:param options: string tokens alternating the driver settings.
		:param direct_session: Another driver object or pyVisa object to reuse the session instead of opening a new session."""
		self._core = Core(resource_name, id_query, reset, RsPulseSeq._driver_options, options, direct_session)
		self._core.driver_version = '2.4.0.0021'
		self._options = options
		self._add_all_global_repcaps()
		self._custom_properties_init()
		self.utilities.default_instrument_setup()
		# noinspection PyTypeChecker
		self._cmd_group = CommandsGroup("ROOT", self._core, None)

	@classmethod
	def from_existing_session(cls, session: object, options: str = None) -> 'RsPulseSeq':
		"""Creates a new RsPulseSeq object with the entered 'session' reused. \n
		:param session: can be another driver or a direct pyvisa session.
		:param options: string tokens alternating the driver settings."""
		# noinspection PyTypeChecker
		return cls(None, False, False, options, session)
		
	@classmethod
	def set_global_logging_target(cls, target) -> None:
		"""Sets global common target stream that each instance can use. To use it, call the following: io.utilities.logger.set_logging_target_global().
		If an instance uses global logging target, it automatically uses the global relative timestamp (if set).
		You can set the target to None to invalidate it."""
		cls._global_logging_target_stream = target

	@classmethod
	def get_global_logging_target(cls):
		"""Returns global common target stream."""
		return cls._global_logging_target_stream

	@classmethod
	def set_global_logging_relative_timestamp(cls, timestamp: datetime) -> None:
		"""Sets global common relative timestamp for log entries. To use it, call the following: io.utilities.logger.set_relative_timestamp_global()"""
		cls._global_logging_relative_timestamp = timestamp

	@classmethod
	def set_global_logging_relative_timestamp_now(cls) -> None:
		"""Sets global common relative timestamp for log entries to this moment.
		To use it, call the following: io.utilities.logger.set_relative_timestamp_global()."""
		cls._global_logging_relative_timestamp = datetime.now()

	@classmethod
	def clear_global_logging_relative_timestamp(cls) -> None:
		"""Clears the global relative timestamp. After this, all the instances using the global relative timestamp continue logging with the absolute timestamps."""
		cls._global_logging_relative_timestamp = None

	@classmethod
	def get_global_logging_relative_timestamp(cls) -> datetime or None:
		"""Returns global common relative timestamp for log entries."""
		return cls._global_logging_relative_timestamp

	def __str__(self) -> str:
		if self._core.io:
			return f"RsPulseSeq session '{self._core.io.resource_name}'"
		else:
			return f"RsPulseSeq with session closed"

	def get_total_execution_time(self) -> timedelta:
		"""Returns total time spent by the library on communicating with the instrument.
		This time is always shorter than get_total_time(), since it does not include gaps between the communication.
		You can reset this counter with reset_time_statistics()."""
		return self._core.io.total_execution_time

	def get_total_time(self) -> timedelta:
		"""Returns total time spent by the library on communicating with the instrument.
		This time is always shorter than get_total_time(), since it does not include gaps between the communication.
		You can reset this counter with reset_time_statistics()."""
		return datetime.now() - self._core.io.total_time_startpoint

	def reset_time_statistics(self) -> None:
		"""Resets all execution and total time counters. Affects the results of get_total_time() and get_total_execution_time()"""
		self._core.io.reset_time_statistics()

	@staticmethod
	def assert_minimum_version(min_version: str) -> None:
		"""Asserts that the driver version fulfills the minimum required version you have entered.
		This way you make sure your installed driver is of the entered version or newer."""
		min_version_list = min_version.split('.')
		curr_version_list = '2.4.0.0021'.split('.')
		count_min = len(min_version_list)
		count_curr = len(curr_version_list)
		count = count_min if count_min < count_curr else count_curr
		for i in range(count):
			minimum = int(min_version_list[i])
			curr = int(curr_version_list[i])
			if curr > minimum:
				break
			if curr < minimum:
				raise RsInstrException(f"Assertion for minimum RsPulseSeq version failed. Current version: '2.4.0.0021', minimum required version: '{min_version}'")

	@staticmethod
	def list_resources(expression: str = '?*::INSTR', visa_select: str = None) -> List[str]:
		"""Finds all the resources defined by the expression
			- '?*' - matches all the available instruments
			- 'USB::?*' - matches all the USB instruments
			- 'TCPIP::192?*' - matches all the LAN instruments with the IP address starting with 192
		:param expression: see the examples in the function
		:param visa_select: optional parameter selecting a specific VISA. Examples: '@ivi', '@rs'
		"""
		rm = VisaSession.get_resource_manager(visa_select)
		resources = rm.list_resources(expression)
		rm.close()
		# noinspection PyTypeChecker
		return resources

	def close(self) -> None:
		"""Closes the active RsPulseSeq session."""
		self._core.io.close()

	def get_session_handle(self) -> object:
		"""Returns the underlying session handle."""
		return self._core.get_session_handle()

	def _add_all_global_repcaps(self) -> None:
		"""Adds all the repcaps defined as global to the instrument's global repcaps dictionary."""

	def _custom_properties_init(self):
		"""Adds all the interfaces that are custom for the driver."""
		from .CustomFiles.utilities import Utilities
		self.utilities = Utilities(self._core)
		from .CustomFiles.events import Events
		self.events = Events(self._core)

	@property
	def adjustment(self):
		"""adjustment commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_adjustment'):
			from .Implementations.Adjustment import Adjustment
			self._adjustment = Adjustment(self._core, self._cmd_group)
		return self._adjustment

	@property
	def antenna(self):
		"""antenna commands group. 1 Sub-classes, 6 commands."""
		if not hasattr(self, '_antenna'):
			from .Implementations.Antenna import Antenna
			self._antenna = Antenna(self._core, self._cmd_group)
		return self._antenna

	@property
	def assignment(self):
		"""assignment commands group. 4 Sub-classes, 0 commands."""
		if not hasattr(self, '_assignment'):
			from .Implementations.Assignment import Assignment
			self._assignment = Assignment(self._core, self._cmd_group)
		return self._assignment

	@property
	def cpanel(self):
		"""cpanel commands group. 6 Sub-classes, 3 commands."""
		if not hasattr(self, '_cpanel'):
			from .Implementations.Cpanel import Cpanel
			self._cpanel = Cpanel(self._core, self._cmd_group)
		return self._cpanel

	@property
	def dialog(self):
		"""dialog commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_dialog'):
			from .Implementations.Dialog import Dialog
			self._dialog = Dialog(self._core, self._cmd_group)
		return self._dialog

	@property
	def dsrc(self):
		"""dsrc commands group. 1 Sub-classes, 7 commands."""
		if not hasattr(self, '_dsrc'):
			from .Implementations.Dsrc import Dsrc
			self._dsrc = Dsrc(self._core, self._cmd_group)
		return self._dsrc

	@property
	def emitter(self):
		"""emitter commands group. 1 Sub-classes, 8 commands."""
		if not hasattr(self, '_emitter'):
			from .Implementations.Emitter import Emitter
			self._emitter = Emitter(self._core, self._cmd_group)
		return self._emitter

	@property
	def generator(self):
		"""generator commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_generator'):
			from .Implementations.Generator import Generator
			self._generator = Generator(self._core, self._cmd_group)
		return self._generator

	@property
	def importPy(self):
		"""importPy commands group. 2 Sub-classes, 0 commands."""
		if not hasattr(self, '_importPy'):
			from .Implementations.ImportPy import ImportPy
			self._importPy = ImportPy(self._core, self._cmd_group)
		return self._importPy

	@property
	def instrument(self):
		"""instrument commands group. 1 Sub-classes, 18 commands."""
		if not hasattr(self, '_instrument'):
			from .Implementations.Instrument import Instrument
			self._instrument = Instrument(self._core, self._cmd_group)
		return self._instrument

	@property
	def ipm(self):
		"""ipm commands group. 9 Sub-classes, 9 commands."""
		if not hasattr(self, '_ipm'):
			from .Implementations.Ipm import Ipm
			self._ipm = Ipm(self._core, self._cmd_group)
		return self._ipm

	@property
	def msgLog(self):
		"""msgLog commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_msgLog'):
			from .Implementations.MsgLog import MsgLog
			self._msgLog = MsgLog(self._core, self._cmd_group)
		return self._msgLog

	@property
	def platform(self):
		"""platform commands group. 1 Sub-classes, 6 commands."""
		if not hasattr(self, '_platform'):
			from .Implementations.Platform import Platform
			self._platform = Platform(self._core, self._cmd_group)
		return self._platform

	@property
	def plot(self):
		"""plot commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_plot'):
			from .Implementations.Plot import Plot
			self._plot = Plot(self._core, self._cmd_group)
		return self._plot

	@property
	def plugin(self):
		"""plugin commands group. 1 Sub-classes, 7 commands."""
		if not hasattr(self, '_plugin'):
			from .Implementations.Plugin import Plugin
			self._plugin = Plugin(self._core, self._cmd_group)
		return self._plugin

	@property
	def preview(self):
		"""preview commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_preview'):
			from .Implementations.Preview import Preview
			self._preview = Preview(self._core, self._cmd_group)
		return self._preview

	@property
	def program(self):
		"""program commands group. 15 Sub-classes, 0 commands."""
		if not hasattr(self, '_program'):
			from .Implementations.Program import Program
			self._program = Program(self._core, self._cmd_group)
		return self._program

	@property
	def pulse(self):
		"""pulse commands group. 9 Sub-classes, 8 commands."""
		if not hasattr(self, '_pulse'):
			from .Implementations.Pulse import Pulse
			self._pulse = Pulse(self._core, self._cmd_group)
		return self._pulse

	@property
	def receiver(self):
		"""receiver commands group. 1 Sub-classes, 7 commands."""
		if not hasattr(self, '_receiver'):
			from .Implementations.Receiver import Receiver
			self._receiver = Receiver(self._core, self._cmd_group)
		return self._receiver

	@property
	def repmanager(self):
		"""repmanager commands group. 1 Sub-classes, 4 commands."""
		if not hasattr(self, '_repmanager'):
			from .Implementations.Repmanager import Repmanager
			self._repmanager = Repmanager(self._core, self._cmd_group)
		return self._repmanager

	@property
	def repository(self):
		"""repository commands group. 2 Sub-classes, 15 commands."""
		if not hasattr(self, '_repository'):
			from .Implementations.Repository import Repository
			self._repository = Repository(self._core, self._cmd_group)
		return self._repository

	@property
	def scan(self):
		"""scan commands group. 10 Sub-classes, 8 commands."""
		if not hasattr(self, '_scan'):
			from .Implementations.Scan import Scan
			self._scan = Scan(self._core, self._cmd_group)
		return self._scan

	@property
	def scenario(self):
		"""scenario commands group. 16 Sub-classes, 13 commands."""
		if not hasattr(self, '_scenario'):
			from .Implementations.Scenario import Scenario
			self._scenario = Scenario(self._core, self._cmd_group)
		return self._scenario

	@property
	def script(self):
		"""script commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_script'):
			from .Implementations.Script import Script
			self._script = Script(self._core, self._cmd_group)
		return self._script

	@property
	def sequence(self):
		"""sequence commands group. 3 Sub-classes, 7 commands."""
		if not hasattr(self, '_sequence'):
			from .Implementations.Sequence import Sequence
			self._sequence = Sequence(self._core, self._cmd_group)
		return self._sequence

	@property
	def setup(self):
		"""setup commands group. 4 Sub-classes, 8 commands."""
		if not hasattr(self, '_setup'):
			from .Implementations.Setup import Setup
			self._setup = Setup(self._core, self._cmd_group)
		return self._setup

	@property
	def status(self):
		"""status commands group. 2 Sub-classes, 0 commands."""
		if not hasattr(self, '_status'):
			from .Implementations.Status import Status
			self._status = Status(self._core, self._cmd_group)
		return self._status

	@property
	def system(self):
		"""system commands group. 1 Sub-classes, 1 commands."""
		if not hasattr(self, '_system'):
			from .Implementations.System import System
			self._system = System(self._core, self._cmd_group)
		return self._system

	@property
	def waveform(self):
		"""waveform commands group. 8 Sub-classes, 8 commands."""
		if not hasattr(self, '_waveform'):
			from .Implementations.Waveform import Waveform
			self._waveform = Waveform(self._core, self._cmd_group)
		return self._waveform

	def clone(self) -> 'RsPulseSeq':
		"""Creates a deep copy of the RsPulseSeq object. Also copies:
			- All the existing Global repeated capability values
			- All the default group repeated capabilities setting \n
		Does not check the *IDN? response, and does not perform Reset.
		After cloning, you can set all the repeated capabilities settings independentely from the original group.
		Calling close() on the new object does not close the original VISA session"""
		cloned = RsPulseSeq.from_existing_session(self.get_session_handle(), self._options)
		self._cmd_group.synchronize_repcaps(cloned)
		
		return cloned

	def restore_all_repcaps_to_default(self) -> None:
		"""Sets all the Group and Global repcaps to their initial values"""
		self._cmd_group.restore_repcaps()



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Direction:
	"""Direction commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("direction", core, parent)

	def get_away(self) -> bool:
		"""SCPI: PLATform:EMITter:DIRection:AWAY \n
		Snippet: value: bool = driver.platform.emitter.direction.get_away() \n
		This command automatically configures the transmission direction of the selected emitter. No effect if emitter is at
		origin. Affects emitters whose position (relative to the origin) has been defined by one of the following methods:
			INTRO_CMD_HELP: Examples of special characters: \n
			- Set X and Y values
			- Set Angle and Radius values
		The transmission direction is configured so that it is directly away from the origin. \n
			:return: away: ON| OFF| 1| 0
		"""
		response = self._core.io.query_str('PLATform:EMITter:DIRection:AWAY?')
		return Conversions.str_to_bool(response)

	def set_away(self, away: bool) -> None:
		"""SCPI: PLATform:EMITter:DIRection:AWAY \n
		Snippet: driver.platform.emitter.direction.set_away(away = False) \n
		This command automatically configures the transmission direction of the selected emitter. No effect if emitter is at
		origin. Affects emitters whose position (relative to the origin) has been defined by one of the following methods:
			INTRO_CMD_HELP: Examples of special characters: \n
			- Set X and Y values
			- Set Angle and Radius values
		The transmission direction is configured so that it is directly away from the origin. \n
			:param away: ON| OFF| 1| 0
		"""
		param = Conversions.bool_to_str(away)
		self._core.io.write(f'PLATform:EMITter:DIRection:AWAY {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from .... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Chirp:
	"""Chirp commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("chirp", core, parent)

	def get_deviation(self) -> float:
		"""SCPI: PULSe:MOP:CHIRp:DEViation \n
		Snippet: value: float = driver.pulse.mop.chirp.get_deviation() \n
		Sets the modulation deviation. \n
			:return: deviation: float Range: 1 to 1e+09
		"""
		response = self._core.io.query_str('PULSe:MOP:CHIRp:DEViation?')
		return Conversions.str_to_float(response)

	def set_deviation(self, deviation: float) -> None:
		"""SCPI: PULSe:MOP:CHIRp:DEViation \n
		Snippet: driver.pulse.mop.chirp.set_deviation(deviation = 1.0) \n
		Sets the modulation deviation. \n
			:param deviation: float Range: 1 to 1e+09
		"""
		param = Conversions.decimal_value_to_str(deviation)
		self._core.io.write(f'PULSe:MOP:CHIRp:DEViation {param}')

	# noinspection PyTypeChecker
	def get_type_py(self) -> enums.ChirpType:
		"""SCPI: PULSe:MOP:CHIRp:TYPE \n
		Snippet: value: enums.ChirpType = driver.pulse.mop.chirp.get_type_py() \n
		Selects the modulation type. \n
			:return: type_py: UP| DOWN| SINE| TRIangular| PIECewise
		"""
		response = self._core.io.query_str('PULSe:MOP:CHIRp:TYPE?')
		return Conversions.str_to_scalar_enum(response, enums.ChirpType)

	def set_type_py(self, type_py: enums.ChirpType) -> None:
		"""SCPI: PULSe:MOP:CHIRp:TYPE \n
		Snippet: driver.pulse.mop.chirp.set_type_py(type_py = enums.ChirpType.DOWN) \n
		Selects the modulation type. \n
			:param type_py: UP| DOWN| SINE| TRIangular| PIECewise
		"""
		param = Conversions.enum_scalar_to_str(type_py, enums.ChirpType)
		self._core.io.write(f'PULSe:MOP:CHIRp:TYPE {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Step:
	"""Step commands group definition. 6 total commands, 0 Subgroups, 6 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("step", core, parent)

	# noinspection PyTypeChecker
	def get_base(self) -> enums.BaseDomainB:
		"""SCPI: IPM:STEP:BASE \n
		Snippet: value: enums.BaseDomainB = driver.ipm.step.get_base() \n
		Sets the IPM profile base and defines how the increments repetition is defined. \n
			:return: base: LENGth| TIME LENGth Steps are repeated several times, as set with the command method RsPulseSeq.Ipm.Step.burst. TIME Steps are repeated for the defined time duration, as set with the command method RsPulseSeq.Ipm.Step.period.
		"""
		response = self._core.io.query_str('IPM:STEP:BASE?')
		return Conversions.str_to_scalar_enum(response, enums.BaseDomainB)

	def set_base(self, base: enums.BaseDomainB) -> None:
		"""SCPI: IPM:STEP:BASE \n
		Snippet: driver.ipm.step.set_base(base = enums.BaseDomainB.LENGth) \n
		Sets the IPM profile base and defines how the increments repetition is defined. \n
			:param base: LENGth| TIME LENGth Steps are repeated several times, as set with the command method RsPulseSeq.Ipm.Step.burst. TIME Steps are repeated for the defined time duration, as set with the command method RsPulseSeq.Ipm.Step.period.
		"""
		param = Conversions.enum_scalar_to_str(base, enums.BaseDomainB)
		self._core.io.write(f'IPM:STEP:BASE {param}')

	def get_burst(self) -> float:
		"""SCPI: IPM:STEP:BURSt \n
		Snippet: value: float = driver.ipm.step.get_burst() \n
		Sets the number of times an increment is repeated. \n
			:return: burst: float Range: 1 to 1000
		"""
		response = self._core.io.query_str('IPM:STEP:BURSt?')
		return Conversions.str_to_float(response)

	def set_burst(self, burst: float) -> None:
		"""SCPI: IPM:STEP:BURSt \n
		Snippet: driver.ipm.step.set_burst(burst = 1.0) \n
		Sets the number of times an increment is repeated. \n
			:param burst: float Range: 1 to 1000
		"""
		param = Conversions.decimal_value_to_str(burst)
		self._core.io.write(f'IPM:STEP:BURSt {param}')

	def get_increment(self) -> float:
		"""SCPI: IPM:STEP:INCRement \n
		Snippet: value: float = driver.ipm.step.get_increment() \n
		Sets the step size. \n
			:return: increment: float Range: -1e+09 to 1e+09
		"""
		response = self._core.io.query_str('IPM:STEP:INCRement?')
		return Conversions.str_to_float(response)

	def set_increment(self, increment: float) -> None:
		"""SCPI: IPM:STEP:INCRement \n
		Snippet: driver.ipm.step.set_increment(increment = 1.0) \n
		Sets the step size. \n
			:param increment: float Range: -1e+09 to 1e+09
		"""
		param = Conversions.decimal_value_to_str(increment)
		self._core.io.write(f'IPM:STEP:INCRement {param}')

	def get_period(self) -> float:
		"""SCPI: IPM:STEP:PERiod \n
		Snippet: value: float = driver.ipm.step.get_period() \n
		Sets how long an increment is repeated. \n
			:return: period: float Range: 1e-09 to 1e+09
		"""
		response = self._core.io.query_str('IPM:STEP:PERiod?')
		return Conversions.str_to_float(response)

	def set_period(self, period: float) -> None:
		"""SCPI: IPM:STEP:PERiod \n
		Snippet: driver.ipm.step.set_period(period = 1.0) \n
		Sets how long an increment is repeated. \n
			:param period: float Range: 1e-09 to 1e+09
		"""
		param = Conversions.decimal_value_to_str(period)
		self._core.io.write(f'IPM:STEP:PERiod {param}')

	def get_start(self) -> float:
		"""SCPI: IPM:STEP:STARt \n
		Snippet: value: float = driver.ipm.step.get_start() \n
		Sets the start value. \n
			:return: start: float Range: -1e+09 to 1e+09
		"""
		response = self._core.io.query_str('IPM:STEP:STARt?')
		return Conversions.str_to_float(response)

	def set_start(self, start: float) -> None:
		"""SCPI: IPM:STEP:STARt \n
		Snippet: driver.ipm.step.set_start(start = 1.0) \n
		Sets the start value. \n
			:param start: float Range: -1e+09 to 1e+09
		"""
		param = Conversions.decimal_value_to_str(start)
		self._core.io.write(f'IPM:STEP:STARt {param}')

	def get_steps(self) -> float:
		"""SCPI: IPM:STEP:STEPs \n
		Snippet: value: float = driver.ipm.step.get_steps() \n
		Sets the number of steps. \n
			:return: steps: float Range: 1 to 10000
		"""
		response = self._core.io.query_str('IPM:STEP:STEPs?')
		return Conversions.str_to_float(response)

	def set_steps(self, steps: float) -> None:
		"""SCPI: IPM:STEP:STEPs \n
		Snippet: driver.ipm.step.set_steps(steps = 1.0) \n
		Sets the number of steps. \n
			:param steps: float Range: 1 to 10000
		"""
		param = Conversions.decimal_value_to_str(steps)
		self._core.io.write(f'IPM:STEP:STEPs {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class NlFm:
	"""NlFm commands group definition. 3 total commands, 0 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("nlFm", core, parent)

	def get_cubic(self) -> float:
		"""SCPI: IMPort:PDW:DATA:NLFM:CUBic \n
		Snippet: value: float = driver.importPy.pdw.data.nlFm.get_cubic() \n
		Queries the pulse parameter. \n
			:return: cubic: float
		"""
		response = self._core.io.query_str('IMPort:PDW:DATA:NLFM:CUBic?')
		return Conversions.str_to_float(response)

	def get_linear(self) -> float:
		"""SCPI: IMPort:PDW:DATA:NLFM:LINear \n
		Snippet: value: float = driver.importPy.pdw.data.nlFm.get_linear() \n
		Queries the pulse parameter. \n
			:return: linear: float
		"""
		response = self._core.io.query_str('IMPort:PDW:DATA:NLFM:LINear?')
		return Conversions.str_to_float(response)

	def get_quadratic(self) -> float:
		"""SCPI: IMPort:PDW:DATA:NLFM:QUADratic \n
		Snippet: value: float = driver.importPy.pdw.data.nlFm.get_quadratic() \n
		Queries the pulse parameter. \n
			:return: quadratic: float
		"""
		response = self._core.io.query_str('IMPort:PDW:DATA:NLFM:QUADratic?')
		return Conversions.str_to_float(response)



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from .....Internal.Utilities import trim_str_response
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Condition:
	"""Condition commands group definition. 4 total commands, 0 Subgroups, 4 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("condition", core, parent)

	# noinspection PyTypeChecker
	def get_type_py(self) -> enums.Condition:
		"""SCPI: SEQuence:ITEM:MARKer:CONDition:TYPE \n
		Snippet: value: enums.Condition = driver.sequence.item.marker.condition.get_type_py() \n
		Sets the sign in the logical condition. \n
			:return: type_py: SMALler| GREater| EQUal| NOTequal
		"""
		response = self._core.io.query_str('SEQuence:ITEM:MARKer:CONDition:TYPE?')
		return Conversions.str_to_scalar_enum(response, enums.Condition)

	def set_type_py(self, type_py: enums.Condition) -> None:
		"""SCPI: SEQuence:ITEM:MARKer:CONDition:TYPE \n
		Snippet: driver.sequence.item.marker.condition.set_type_py(type_py = enums.Condition.EQUal) \n
		Sets the sign in the logical condition. \n
			:param type_py: SMALler| GREater| EQUal| NOTequal
		"""
		param = Conversions.enum_scalar_to_str(type_py, enums.Condition)
		self._core.io.write(f'SEQuence:ITEM:MARKer:CONDition:TYPE {param}')

	def get_string_value(self) -> str:
		"""SCPI: SEQuence:ITEM:MARKer:CONDition:VALue \n
		Snippet: value: str = driver.sequence.item.marker.condition.get_string_value() \n
		Sets the numerical value used with the comparison. \n
			:return: value: string
		"""
		response = self._core.io.query_str('SEQuence:ITEM:MARKer:CONDition:VALue?')
		return trim_str_response(response)

	def set_string_value(self, value: str) -> None:
		"""SCPI: SEQuence:ITEM:MARKer:CONDition:VALue \n
		Snippet: driver.sequence.item.marker.condition.set_string_value(value = '1') \n
		Sets the numerical value used with the comparison. \n
			:param value: string
		"""
		param = Conversions.value_to_quoted_str(value)
		self._core.io.write(f'SEQuence:ITEM:MARKer:CONDition:VALue {param}')

	def get_variable(self) -> str:
		"""SCPI: SEQuence:ITEM:MARKer:CONDition:VARiable \n
		Snippet: value: str = driver.sequence.item.marker.condition.get_variable() \n
		Defines the value that is compared with the fixed values set with the command method RsPulseSeq.Sequence.Item.Marker.
		Condition.stringValue. \n
			:return: variable: string
		"""
		response = self._core.io.query_str('SEQuence:ITEM:MARKer:CONDition:VARiable?')
		return trim_str_response(response)

	def set_variable(self, variable: str) -> None:
		"""SCPI: SEQuence:ITEM:MARKer:CONDition:VARiable \n
		Snippet: driver.sequence.item.marker.condition.set_variable(variable = '1') \n
		Defines the value that is compared with the fixed values set with the command method RsPulseSeq.Sequence.Item.Marker.
		Condition.stringValue. \n
			:param variable: string
		"""
		param = Conversions.value_to_quoted_str(variable)
		self._core.io.write(f'SEQuence:ITEM:MARKer:CONDition:VARiable {param}')

	def get_value(self) -> float:
		"""SCPI: SEQuence:ITEM:MARKer:CONDition \n
		Snippet: value: float = driver.sequence.item.marker.condition.get_value() \n
		Enables up to four markers of the corresponding type. \n
			:return: condition: No help available
		"""
		response = self._core.io.query_str('SEQuence:ITEM:MARKer:CONDition?')
		return Conversions.str_to_float(response)

	def set_value(self, condition: float) -> None:
		"""SCPI: SEQuence:ITEM:MARKer:CONDition \n
		Snippet: driver.sequence.item.marker.condition.set_value(condition = 1.0) \n
		Enables up to four markers of the corresponding type. \n
			:param condition: float See Table 'Setting parameter as function of the marker states'. Range: 0 to 65535
		"""
		param = Conversions.decimal_value_to_str(condition)
		self._core.io.write(f'SEQuence:ITEM:MARKer:CONDition {param}')



from ..Internal.Core import Core
from ..Internal.CommandsGroup import CommandsGroup
from ..Internal import Conversions
from .. import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Generator:
	"""Generator commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("generator", core, parent)

	# noinspection PyTypeChecker
	def get_type_py(self) -> enums.GeneratorType:
		"""SCPI: GENerator:TYPE \n
		Snippet: value: enums.GeneratorType = driver.generator.get_type_py() \n
		Sets the instrument's type. \n
			:return: type_py: SMU| SMM| SMBB| SW| SGT| SMBV| SMJ| SMW SMBB designates R&S SMBV100B.
		"""
		response = self._core.io.query_str('GENerator:TYPE?')
		return Conversions.str_to_scalar_enum(response, enums.GeneratorType)

	def set_type_py(self, type_py: enums.GeneratorType) -> None:
		"""SCPI: GENerator:TYPE \n
		Snippet: driver.generator.set_type_py(type_py = enums.GeneratorType.SGT) \n
		Sets the instrument's type. \n
			:param type_py: SMU| SMM| SMBB| SW| SGT| SMBV| SMJ| SMW SMBB designates R&S SMBV100B.
		"""
		param = Conversions.enum_scalar_to_str(type_py, enums.GeneratorType)
		self._core.io.write(f'GENerator:TYPE {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ...Internal.Utilities import trim_str_response
from ... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Ipm:
	"""Ipm commands group definition. 66 total commands, 9 Subgroups, 9 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("ipm", core, parent)

	@property
	def binomial(self):
		"""binomial commands group. 0 Sub-classes, 3 commands."""
		if not hasattr(self, '_binomial'):
			from .Binomial import Binomial
			self._binomial = Binomial(self._core, self._cmd_group)
		return self._binomial

	@property
	def listPy(self):
		"""listPy commands group. 2 Sub-classes, 4 commands."""
		if not hasattr(self, '_listPy'):
			from .ListPy import ListPy
			self._listPy = ListPy(self._core, self._cmd_group)
		return self._listPy

	@property
	def plugin(self):
		"""plugin commands group. 1 Sub-classes, 1 commands."""
		if not hasattr(self, '_plugin'):
			from .Plugin import Plugin
			self._plugin = Plugin(self._core, self._cmd_group)
		return self._plugin

	@property
	def random(self):
		"""random commands group. 3 Sub-classes, 1 commands."""
		if not hasattr(self, '_random'):
			from .Random import Random
			self._random = Random(self._core, self._cmd_group)
		return self._random

	@property
	def rlist(self):
		"""rlist commands group. 0 Sub-classes, 4 commands."""
		if not hasattr(self, '_rlist'):
			from .Rlist import Rlist
			self._rlist = Rlist(self._core, self._cmd_group)
		return self._rlist

	@property
	def rstep(self):
		"""rstep commands group. 1 Sub-classes, 3 commands."""
		if not hasattr(self, '_rstep'):
			from .Rstep import Rstep
			self._rstep = Rstep(self._core, self._cmd_group)
		return self._rstep

	@property
	def shape(self):
		"""shape commands group. 0 Sub-classes, 4 commands."""
		if not hasattr(self, '_shape'):
			from .Shape import Shape
			self._shape = Shape(self._core, self._cmd_group)
		return self._shape

	@property
	def step(self):
		"""step commands group. 0 Sub-classes, 6 commands."""
		if not hasattr(self, '_step'):
			from .Step import Step
			self._step = Step(self._core, self._cmd_group)
		return self._step

	@property
	def waveform(self):
		"""waveform commands group. 0 Sub-classes, 7 commands."""
		if not hasattr(self, '_waveform'):
			from .Waveform import Waveform
			self._waveform = Waveform(self._core, self._cmd_group)
		return self._waveform

	def get_catalog(self) -> str:
		"""SCPI: IPM:CATalog \n
		Snippet: value: str = driver.ipm.get_catalog() \n
		Queries the available repository elements in the database. \n
			:return: catalog: string
		"""
		response = self._core.io.query_str('IPM:CATalog?')
		return trim_str_response(response)

	def get_comment(self) -> str:
		"""SCPI: IPM:COMMent \n
		Snippet: value: str = driver.ipm.get_comment() \n
		Adds a description to the selected repository element. \n
			:return: comment: string
		"""
		response = self._core.io.query_str('IPM:COMMent?')
		return trim_str_response(response)

	def set_comment(self, comment: str) -> None:
		"""SCPI: IPM:COMMent \n
		Snippet: driver.ipm.set_comment(comment = '1') \n
		Adds a description to the selected repository element. \n
			:param comment: string
		"""
		param = Conversions.value_to_quoted_str(comment)
		self._core.io.write(f'IPM:COMMent {param}')

	def set_create(self, create: str) -> None:
		"""SCPI: IPM:CREate \n
		Snippet: driver.ipm.set_create(create = '1') \n
		Creates a repository element with the selected name. \n
			:param create: string Must be unique for the particular type of repository elements. May contain empty spaces.
		"""
		param = Conversions.value_to_quoted_str(create)
		self._core.io.write(f'IPM:CREate {param}')

	def get_equation(self) -> str:
		"""SCPI: IPM:EQUation \n
		Snippet: value: str = driver.ipm.get_equation() \n
		Defines the IPM shape as a function. \n
			:return: equation: string
		"""
		response = self._core.io.query_str('IPM:EQUation?')
		return trim_str_response(response)

	def set_equation(self, equation: str) -> None:
		"""SCPI: IPM:EQUation \n
		Snippet: driver.ipm.set_equation(equation = '1') \n
		Defines the IPM shape as a function. \n
			:param equation: string
		"""
		param = Conversions.value_to_quoted_str(equation)
		self._core.io.write(f'IPM:EQUation {param}')

	def get_name(self) -> str:
		"""SCPI: IPM:NAME \n
		Snippet: value: str = driver.ipm.get_name() \n
		Renames the selected repository element. \n
			:return: name: string Must be unique for the particular type of repository elements. May contain empty spaces.
		"""
		response = self._core.io.query_str('IPM:NAME?')
		return trim_str_response(response)

	def set_name(self, name: str) -> None:
		"""SCPI: IPM:NAME \n
		Snippet: driver.ipm.set_name(name = '1') \n
		Renames the selected repository element. \n
			:param name: string Must be unique for the particular type of repository elements. May contain empty spaces.
		"""
		param = Conversions.value_to_quoted_str(name)
		self._core.io.write(f'IPM:NAME {param}')

	def set_remove(self, remove: str) -> None:
		"""SCPI: IPM:REMove \n
		Snippet: driver.ipm.set_remove(remove = '1') \n
		Removes the selected element from the workspace. The element must not reference any child elements. Remove the referenced
		elements first. \n
			:param remove: No help available
		"""
		param = Conversions.value_to_quoted_str(remove)
		self._core.io.write(f'IPM:REMove {param}')

	def get_select(self) -> str:
		"""SCPI: IPM:SELect \n
		Snippet: value: str = driver.ipm.get_select() \n
		Selects the repository element to which the subsequent commands apply. \n
			:return: select: string Element name, as defined with the ...:CREate or ...:NAME command. To query the existing elements, use the ...:CATalog? command. For example, method RsPulseSeq.Repository.catalog.
		"""
		response = self._core.io.query_str('IPM:SELect?')
		return trim_str_response(response)

	def set_select(self, select: str) -> None:
		"""SCPI: IPM:SELect \n
		Snippet: driver.ipm.set_select(select = '1') \n
		Selects the repository element to which the subsequent commands apply. \n
			:param select: string Element name, as defined with the ...:CREate or ...:NAME command. To query the existing elements, use the ...:CATalog? command. For example, method RsPulseSeq.Repository.catalog.
		"""
		param = Conversions.value_to_quoted_str(select)
		self._core.io.write(f'IPM:SELect {param}')

	# noinspection PyTypeChecker
	def get_type_py(self) -> enums.IpmType:
		"""SCPI: IPM:TYPE \n
		Snippet: value: enums.IpmType = driver.ipm.get_type_py() \n
		Sets the shape of the profile. \n
			:return: type_py: STEPs| WAVeform| RLISt| LIST| SHAPe| RANDom| EQUation| PLUGin| RSTep| BINomial
		"""
		response = self._core.io.query_str('IPM:TYPE?')
		return Conversions.str_to_scalar_enum(response, enums.IpmType)

	def set_type_py(self, type_py: enums.IpmType) -> None:
		"""SCPI: IPM:TYPE \n
		Snippet: driver.ipm.set_type_py(type_py = enums.IpmType.BINomial) \n
		Sets the shape of the profile. \n
			:param type_py: STEPs| WAVeform| RLISt| LIST| SHAPe| RANDom| EQUation| PLUGin| RSTep| BINomial
		"""
		param = Conversions.enum_scalar_to_str(type_py, enums.IpmType)
		self._core.io.write(f'IPM:TYPE {param}')

	# noinspection PyTypeChecker
	def get_unit(self) -> enums.Units:
		"""SCPI: IPM:UNIT \n
		Snippet: value: enums.Units = driver.ipm.get_unit() \n
		Sets the units of the profile. \n
			:return: unit: NONE| SEConds| HERTz| DB| DEGRees| PERCent
		"""
		response = self._core.io.query_str('IPM:UNIT?')
		return Conversions.str_to_scalar_enum(response, enums.Units)

	def set_unit(self, unit: enums.Units) -> None:
		"""SCPI: IPM:UNIT \n
		Snippet: driver.ipm.set_unit(unit = enums.Units.DB) \n
		Sets the units of the profile. \n
			:param unit: NONE| SEConds| HERTz| DB| DEGRees| PERCent
		"""
		param = Conversions.enum_scalar_to_str(unit, enums.Units)
		self._core.io.write(f'IPM:UNIT {param}')

	def clone(self) -> 'Ipm':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = Ipm(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from ....Internal.Utilities import trim_str_response
from .... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Item:
	"""Item commands group definition. 10 total commands, 2 Subgroups, 6 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("item", core, parent)

	@property
	def add(self):
		"""add commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_add'):
			from .Add import Add
			self._add = Add(self._core, self._cmd_group)
		return self._add

	@property
	def prbs(self):
		"""prbs commands group. 1 Sub-classes, 1 commands."""
		if not hasattr(self, '_prbs'):
			from .Prbs import Prbs
			self._prbs = Prbs(self._core, self._cmd_group)
		return self._prbs

	def get_bits(self) -> float:
		"""SCPI: DSRC:ITEM:BITS \n
		Snippet: value: float = driver.dsrc.item.get_bits() \n
		Sets the length of the selected item in bits. \n
			:return: bits: float Range: 0 to 4096
		"""
		response = self._core.io.query_str('DSRC:ITEM:BITS?')
		return Conversions.str_to_float(response)

	def set_bits(self, bits: float) -> None:
		"""SCPI: DSRC:ITEM:BITS \n
		Snippet: driver.dsrc.item.set_bits(bits = 1.0) \n
		Sets the length of the selected item in bits. \n
			:param bits: float Range: 0 to 4096
		"""
		param = Conversions.decimal_value_to_str(bits)
		self._core.io.write(f'DSRC:ITEM:BITS {param}')

	def get_data(self) -> str:
		"""SCPI: DSRC:ITEM:DATA \n
		Snippet: value: str = driver.dsrc.item.get_data() \n
		Sets the user defined data pattern. \n
			:return: data: string
		"""
		response = self._core.io.query_str('DSRC:ITEM:DATA?')
		return trim_str_response(response)

	def set_data(self, data: str) -> None:
		"""SCPI: DSRC:ITEM:DATA \n
		Snippet: driver.dsrc.item.set_data(data = '1') \n
		Sets the user defined data pattern. \n
			:param data: string
		"""
		param = Conversions.value_to_quoted_str(data)
		self._core.io.write(f'DSRC:ITEM:DATA {param}')

	def delete(self, delete: float) -> None:
		"""SCPI: DSRC:ITEM:DELete \n
		Snippet: driver.dsrc.item.delete(delete = 1.0) \n
		Deletes the particular item. \n
			:param delete: float
		"""
		param = Conversions.decimal_value_to_str(delete)
		self._core.io.write(f'DSRC:ITEM:DELete {param}')

	# noinspection PyTypeChecker
	def get_pattern(self) -> enums.ItemPattern:
		"""SCPI: DSRC:ITEM:PATTern \n
		Snippet: value: enums.ItemPattern = driver.dsrc.item.get_pattern() \n
		Sets the data pattern of the selected item. \n
			:return: pattern: ZERO| ONE| ALT| R2A| R2B| R3| R4A| R4B| R5| R7| R11| R13 ZERO|ONE Binary 0 and 1 ALT Variable bit strings ('1010') with alternating 0 and 1 and a maximum length of 999 bits R2A|R2B|R3|R4A|R4B|R5|R7|R11|R13 Barker codes
		"""
		response = self._core.io.query_str('DSRC:ITEM:PATTern?')
		return Conversions.str_to_scalar_enum(response, enums.ItemPattern)

	def set_pattern(self, pattern: enums.ItemPattern) -> None:
		"""SCPI: DSRC:ITEM:PATTern \n
		Snippet: driver.dsrc.item.set_pattern(pattern = enums.ItemPattern.ALT) \n
		Sets the data pattern of the selected item. \n
			:param pattern: ZERO| ONE| ALT| R2A| R2B| R3| R4A| R4B| R5| R7| R11| R13 ZERO|ONE Binary 0 and 1 ALT Variable bit strings ('1010') with alternating 0 and 1 and a maximum length of 999 bits R2A|R2B|R3|R4A|R4B|R5|R7|R11|R13 Barker codes
		"""
		param = Conversions.enum_scalar_to_str(pattern, enums.ItemPattern)
		self._core.io.write(f'DSRC:ITEM:PATTern {param}')

	def get_select(self) -> float:
		"""SCPI: DSRC:ITEM:SELect \n
		Snippet: value: float = driver.dsrc.item.get_select() \n
		Selects the item to which the subsequent commands apply. \n
			:return: select: float Item number within the range 1 to ...:COUNt. For example, method RsPulseSeq.Sequence.Item.count. Range: 1 to 4096
		"""
		response = self._core.io.query_str('DSRC:ITEM:SELect?')
		return Conversions.str_to_float(response)

	def set_select(self, select: float) -> None:
		"""SCPI: DSRC:ITEM:SELect \n
		Snippet: driver.dsrc.item.set_select(select = 1.0) \n
		Selects the item to which the subsequent commands apply. \n
			:param select: float Item number within the range 1 to ...:COUNt. For example, method RsPulseSeq.Sequence.Item.count. Range: 1 to 4096
		"""
		param = Conversions.decimal_value_to_str(select)
		self._core.io.write(f'DSRC:ITEM:SELect {param}')

	# noinspection PyTypeChecker
	def get_type_py(self) -> enums.ItemTypeB:
		"""SCPI: DSRC:ITEM:TYPE \n
		Snippet: value: enums.ItemTypeB = driver.dsrc.item.get_type_py() \n
		Sets the data type of selected item. \n
			:return: type_py: PATTern| PRBS| USER
		"""
		response = self._core.io.query_str('DSRC:ITEM:TYPE?')
		return Conversions.str_to_scalar_enum(response, enums.ItemTypeB)

	def set_type_py(self, type_py: enums.ItemTypeB) -> None:
		"""SCPI: DSRC:ITEM:TYPE \n
		Snippet: driver.dsrc.item.set_type_py(type_py = enums.ItemTypeB.PATTern) \n
		Sets the data type of selected item. \n
			:param type_py: PATTern| PRBS| USER
		"""
		param = Conversions.enum_scalar_to_str(type_py, enums.ItemTypeB)
		self._core.io.write(f'DSRC:ITEM:TYPE {param}')

	def clone(self) -> 'Item':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = Item(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Time:
	"""Time commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("time", core, parent)

	def get_start(self) -> float:
		"""SCPI: PULSe:MOP:EXCLude:TIME:STARt \n
		Snippet: value: float = driver.pulse.mop.exclude.time.get_start() \n
		Sets a time span to be excluded at the beginning and at the end of the pulse. \n
			:return: start: No help available
		"""
		response = self._core.io.query_str('PULSe:MOP:EXCLude:TIME:STARt?')
		return Conversions.str_to_float(response)

	def set_start(self, start: float) -> None:
		"""SCPI: PULSe:MOP:EXCLude:TIME:STARt \n
		Snippet: driver.pulse.mop.exclude.time.set_start(start = 1.0) \n
		Sets a time span to be excluded at the beginning and at the end of the pulse. \n
			:param start: float Range: 0 to 5e-07
		"""
		param = Conversions.decimal_value_to_str(start)
		self._core.io.write(f'PULSe:MOP:EXCLude:TIME:STARt {param}')

	def get_stop(self) -> float:
		"""SCPI: PULSe:MOP:EXCLude:TIME:STOP \n
		Snippet: value: float = driver.pulse.mop.exclude.time.get_stop() \n
		Sets a time span to be excluded at the beginning and at the end of the pulse. \n
			:return: stop: float Range: 0 to 5e-07
		"""
		response = self._core.io.query_str('PULSe:MOP:EXCLude:TIME:STOP?')
		return Conversions.str_to_float(response)

	def set_stop(self, stop: float) -> None:
		"""SCPI: PULSe:MOP:EXCLude:TIME:STOP \n
		Snippet: driver.pulse.mop.exclude.time.set_stop(stop = 1.0) \n
		Sets a time span to be excluded at the beginning and at the end of the pulse. \n
			:param stop: float Range: 0 to 5e-07
		"""
		param = Conversions.decimal_value_to_str(stop)
		self._core.io.write(f'PULSe:MOP:EXCLude:TIME:STOP {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Time:
	"""Time commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("time", core, parent)

	def get_start(self) -> float:
		"""SCPI: IMPort:VIEW:TIME:STARt \n
		Snippet: value: float = driver.importPy.view.time.get_start() \n
		Sets the start line displayed on the page. \n
			:return: start: float
		"""
		response = self._core.io.query_str('IMPort:VIEW:TIME:STARt?')
		return Conversions.str_to_float(response)

	def set_start(self, start: float) -> None:
		"""SCPI: IMPort:VIEW:TIME:STARt \n
		Snippet: driver.importPy.view.time.set_start(start = 1.0) \n
		Sets the start line displayed on the page. \n
			:param start: float
		"""
		param = Conversions.decimal_value_to_str(start)
		self._core.io.write(f'IMPort:VIEW:TIME:STARt {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from .....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Dsrc:
	"""Dsrc commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("dsrc", core, parent)

	def get_reset(self) -> bool:
		"""SCPI: PULSe:MOP:DATA:DSRC:RESet \n
		Snippet: value: bool = driver.pulse.mop.data.dsrc.get_reset() \n
		Resets the data source at the end of the pulse. \n
			:return: reset: ON| OFF| 1| 0
		"""
		response = self._core.io.query_str('PULSe:MOP:DATA:DSRC:RESet?')
		return Conversions.str_to_bool(response)

	def set_reset(self, reset: bool) -> None:
		"""SCPI: PULSe:MOP:DATA:DSRC:RESet \n
		Snippet: driver.pulse.mop.data.dsrc.set_reset(reset = False) \n
		Resets the data source at the end of the pulse. \n
			:param reset: ON| OFF| 1| 0
		"""
		param = Conversions.bool_to_str(reset)
		self._core.io.write(f'PULSe:MOP:DATA:DSRC:RESet {param}')

	def get_value(self) -> str:
		"""SCPI: PULSe:MOP:DATA:DSRC \n
		Snippet: value: str = driver.pulse.mop.data.dsrc.get_value() \n
		Selects the data source for the modulation, see method RsPulseSeq.Dsrc.create. \n
			:return: dsrc: string
		"""
		response = self._core.io.query_str('PULSe:MOP:DATA:DSRC?')
		return trim_str_response(response)

	def set_value(self, dsrc: str) -> None:
		"""SCPI: PULSe:MOP:DATA:DSRC \n
		Snippet: driver.pulse.mop.data.dsrc.set_value(dsrc = '1') \n
		Selects the data source for the modulation, see method RsPulseSeq.Dsrc.create. \n
			:param dsrc: string
		"""
		param = Conversions.value_to_quoted_str(dsrc)
		self._core.io.write(f'PULSe:MOP:DATA:DSRC {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Waypoint:
	"""Waypoint commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("waypoint", core, parent)

	def clear(self) -> None:
		"""SCPI: SCENario:LOCalized:LOCation:WAYPoint:CLEar \n
		Snippet: driver.scenario.localized.location.waypoint.clear() \n
		Discards the selected file. \n
		"""
		self._core.io.write(f'SCENario:LOCalized:LOCation:WAYPoint:CLEar')

	def clear_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: SCENario:LOCalized:LOCation:WAYPoint:CLEar \n
		Snippet: driver.scenario.localized.location.waypoint.clear_with_opc() \n
		Discards the selected file. \n
		Same as clear, but waits for the operation to complete before continuing further. Use the RsPulseSeq.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'SCENario:LOCalized:LOCation:WAYPoint:CLEar', opc_timeout_ms)



from ..Internal.Core import Core
from ..Internal.CommandsGroup import CommandsGroup
from ..Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Preview:
	"""Preview commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("preview", core, parent)

	def get_position(self) -> str:
		"""SCPI: PREView:POSition \n
		Snippet: value: str = driver.preview.get_position() \n
		If movement is enabled, queries the current positions of the Tx items. \n
			:return: position: string Semicolon-separated string with the format: TIME=time_from_simulation_start; ID=Tx item ID;NAME=Tx item alias name;DIST=distancekm;LEVATT=Level at Rx origindBm;AZI=Azimuthdeg;ELEV=Elevationdeg;N=Northkm;E=Eastkm;H=Heightkm;
		"""
		response = self._core.io.query_str('PREView:POSition?')
		return trim_str_response(response)



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Cpanel:
	"""Cpanel commands group definition. 25 total commands, 6 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("cpanel", core, parent)

	@property
	def mute(self):
		"""mute commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_mute'):
			from .Mute import Mute
			self._mute = Mute(self._core, self._cmd_group)
		return self._mute

	@property
	def refresh(self):
		"""refresh commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_refresh'):
			from .Refresh import Refresh
			self._refresh = Refresh(self._core, self._cmd_group)
		return self._refresh

	@property
	def scenario(self):
		"""scenario commands group. 2 Sub-classes, 0 commands."""
		if not hasattr(self, '_scenario'):
			from .Scenario import Scenario
			self._scenario = Scenario(self._core, self._cmd_group)
		return self._scenario

	@property
	def unmute(self):
		"""unmute commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_unmute'):
			from .Unmute import Unmute
			self._unmute = Unmute(self._core, self._cmd_group)
		return self._unmute

	@property
	def unused(self):
		"""unused commands group. 1 Sub-classes, 6 commands."""
		if not hasattr(self, '_unused'):
			from .Unused import Unused
			self._unused = Unused(self._core, self._cmd_group)
		return self._unused

	@property
	def used(self):
		"""used commands group. 1 Sub-classes, 5 commands."""
		if not hasattr(self, '_used'):
			from .Used import Used
			self._used = Used(self._core, self._cmd_group)
		return self._used

	def activate(self) -> None:
		"""SCPI: CPANel:ACTivate \n
		Snippet: driver.cpanel.activate() \n
		Activates and deactivates the remote control of the control panel. Further CPANel:... commands cannot be executed, if the
		remote control is not active. After configuration, always deactivate the remote control of the control panel. \n
		"""
		self._core.io.write(f'CPANel:ACTivate')

	def activate_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: CPANel:ACTivate \n
		Snippet: driver.cpanel.activate_with_opc() \n
		Activates and deactivates the remote control of the control panel. Further CPANel:... commands cannot be executed, if the
		remote control is not active. After configuration, always deactivate the remote control of the control panel. \n
		Same as activate, but waits for the operation to complete before continuing further. Use the RsPulseSeq.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'CPANel:ACTivate', opc_timeout_ms)

	def deactivate(self) -> None:
		"""SCPI: CPANel:DEACtivate \n
		Snippet: driver.cpanel.deactivate() \n
		Activates and deactivates the remote control of the control panel. Further CPANel:... commands cannot be executed, if the
		remote control is not active. After configuration, always deactivate the remote control of the control panel. \n
		"""
		self._core.io.write(f'CPANel:DEACtivate')

	def deactivate_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: CPANel:DEACtivate \n
		Snippet: driver.cpanel.deactivate_with_opc() \n
		Activates and deactivates the remote control of the control panel. Further CPANel:... commands cannot be executed, if the
		remote control is not active. After configuration, always deactivate the remote control of the control panel. \n
		Same as deactivate, but waits for the operation to complete before continuing further. Use the RsPulseSeq.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'CPANel:DEACtivate', opc_timeout_ms)

	def get_setup(self) -> str:
		"""SCPI: CPANel:SETup \n
		Snippet: value: str = driver.cpanel.get_setup() \n
		Queries the name of the setup selected in the 'Signal Generators' dialog. \n
			:return: setup: string
		"""
		response = self._core.io.query_str('CPANel:SETup?')
		return trim_str_response(response)

	def clone(self) -> 'Cpanel':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = Cpanel(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from ....Internal.Utilities import trim_str_response
from .... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Ftp:
	"""Ftp commands group definition. 4 total commands, 0 Subgroups, 4 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("ftp", core, parent)

	# noinspection PyTypeChecker
	def get_block_size(self) -> enums.BlockSize:
		"""SCPI: PROGram:TRANsfer:FTP:BLOCksize \n
		Snippet: value: enums.BlockSize = driver.program.transfer.ftp.get_block_size() \n
		Block size used for data transfer. \n
			:return: block_size: 16K| 32K| 64K| 1M| 2M| 4M
		"""
		response = self._core.io.query_str('PROGram:TRANsfer:FTP:BLOCksize?')
		return Conversions.str_to_scalar_enum(response, enums.BlockSize)

	def set_block_size(self, block_size: enums.BlockSize) -> None:
		"""SCPI: PROGram:TRANsfer:FTP:BLOCksize \n
		Snippet: driver.program.transfer.ftp.set_block_size(block_size = enums.BlockSize._16K) \n
		Block size used for data transfer. \n
			:param block_size: 16K| 32K| 64K| 1M| 2M| 4M
		"""
		param = Conversions.enum_scalar_to_str(block_size, enums.BlockSize)
		self._core.io.write(f'PROGram:TRANsfer:FTP:BLOCksize {param}')

	def get_enable(self) -> bool:
		"""SCPI: PROGram:TRANsfer:FTP:ENABle \n
		Snippet: value: bool = driver.program.transfer.ftp.get_enable() \n
		Enables an FTP data transfer for transfer of large files to the processing instrument. \n
			:return: enable: ON| OFF| 1| 0
		"""
		response = self._core.io.query_str('PROGram:TRANsfer:FTP:ENABle?')
		return Conversions.str_to_bool(response)

	def set_enable(self, enable: bool) -> None:
		"""SCPI: PROGram:TRANsfer:FTP:ENABle \n
		Snippet: driver.program.transfer.ftp.set_enable(enable = False) \n
		Enables an FTP data transfer for transfer of large files to the processing instrument. \n
			:param enable: ON| OFF| 1| 0
		"""
		param = Conversions.bool_to_str(enable)
		self._core.io.write(f'PROGram:TRANsfer:FTP:ENABle {param}')

	def get_passwd(self) -> str:
		"""SCPI: PROGram:TRANsfer:FTP:PASSwd \n
		Snippet: value: str = driver.program.transfer.ftp.get_passwd() \n
		No command help available \n
			:return: passwd: string
		"""
		response = self._core.io.query_str('PROGram:TRANsfer:FTP:PASSwd?')
		return trim_str_response(response)

	def set_passwd(self, passwd: str) -> None:
		"""SCPI: PROGram:TRANsfer:FTP:PASSwd \n
		Snippet: driver.program.transfer.ftp.set_passwd(passwd = '1') \n
		No command help available \n
			:param passwd: string
		"""
		param = Conversions.value_to_quoted_str(passwd)
		self._core.io.write(f'PROGram:TRANsfer:FTP:PASSwd {param}')

	def get_uname(self) -> str:
		"""SCPI: PROGram:TRANsfer:FTP:UNAMe \n
		Snippet: value: str = driver.program.transfer.ftp.get_uname() \n
		Sets the user name and password of the processing instrument. \n
			:return: uname: string
		"""
		response = self._core.io.query_str('PROGram:TRANsfer:FTP:UNAMe?')
		return trim_str_response(response)

	def set_uname(self, uname: str) -> None:
		"""SCPI: PROGram:TRANsfer:FTP:UNAMe \n
		Snippet: driver.program.transfer.ftp.set_uname(uname = '1') \n
		Sets the user name and password of the processing instrument. \n
			:param uname: string
		"""
		param = Conversions.value_to_quoted_str(uname)
		self._core.io.write(f'PROGram:TRANsfer:FTP:UNAMe {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Ask:
	"""Ask commands group definition. 3 total commands, 0 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("ask", core, parent)

	def get_invert(self) -> bool:
		"""SCPI: PULSe:MOP:ASK:INVert \n
		Snippet: value: bool = driver.pulse.mop.ask.get_invert() \n
		Inverts the modulation. \n
			:return: invert: ON| OFF| 1| 0
		"""
		response = self._core.io.query_str('PULSe:MOP:ASK:INVert?')
		return Conversions.str_to_bool(response)

	def set_invert(self, invert: bool) -> None:
		"""SCPI: PULSe:MOP:ASK:INVert \n
		Snippet: driver.pulse.mop.ask.set_invert(invert = False) \n
		Inverts the modulation. \n
			:param invert: ON| OFF| 1| 0
		"""
		param = Conversions.bool_to_str(invert)
		self._core.io.write(f'PULSe:MOP:ASK:INVert {param}')

	def get_mdepth(self) -> float:
		"""SCPI: PULSe:MOP:ASK:MDEPth \n
		Snippet: value: float = driver.pulse.mop.ask.get_mdepth() \n
		Sets the modulation depth. \n
			:return: mdepth: float Range: 0 to 100, Unit: percent
		"""
		response = self._core.io.query_str('PULSe:MOP:ASK:MDEPth?')
		return Conversions.str_to_float(response)

	def set_mdepth(self, mdepth: float) -> None:
		"""SCPI: PULSe:MOP:ASK:MDEPth \n
		Snippet: driver.pulse.mop.ask.set_mdepth(mdepth = 1.0) \n
		Sets the modulation depth. \n
			:param mdepth: float Range: 0 to 100, Unit: percent
		"""
		param = Conversions.decimal_value_to_str(mdepth)
		self._core.io.write(f'PULSe:MOP:ASK:MDEPth {param}')

	def get_symbol_rate(self) -> float:
		"""SCPI: PULSe:MOP:ASK:SRATe \n
		Snippet: value: float = driver.pulse.mop.ask.get_symbol_rate() \n
		Sets the symbol rate. \n
			:return: srate: float Range: 1 to 1e+09
		"""
		response = self._core.io.query_str('PULSe:MOP:ASK:SRATe?')
		return Conversions.str_to_float(response)

	def set_symbol_rate(self, srate: float) -> None:
		"""SCPI: PULSe:MOP:ASK:SRATe \n
		Snippet: driver.pulse.mop.ask.set_symbol_rate(srate = 1.0) \n
		Sets the symbol rate. \n
			:param srate: float Range: 1 to 1e+09
		"""
		param = Conversions.decimal_value_to_str(srate)
		self._core.io.write(f'PULSe:MOP:ASK:SRATe {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from ....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class NlCirp:
	"""NlCirp commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("nlCirp", core, parent)

	def get_equation(self) -> str:
		"""SCPI: PULSe:MOP:NLCHirp:EQUation \n
		Snippet: value: str = driver.pulse.mop.nlCirp.get_equation() \n
		Determines the chirp mathematically. \n
			:return: equation: string
		"""
		response = self._core.io.query_str('PULSe:MOP:NLCHirp:EQUation?')
		return trim_str_response(response)

	def set_equation(self, equation: str) -> None:
		"""SCPI: PULSe:MOP:NLCHirp:EQUation \n
		Snippet: driver.pulse.mop.nlCirp.set_equation(equation = '1') \n
		Determines the chirp mathematically. \n
			:param equation: string
		"""
		param = Conversions.value_to_quoted_str(equation)
		self._core.io.write(f'PULSe:MOP:NLCHirp:EQUation {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from ....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Cpdw:
	"""Cpdw commands group definition. 22 total commands, 2 Subgroups, 13 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("cpdw", core, parent)

	@property
	def add(self):
		"""add commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_add'):
			from .Add import Add
			self._add = Add(self._core, self._cmd_group)
		return self._add

	@property
	def group(self):
		"""group commands group. 1 Sub-classes, 7 commands."""
		if not hasattr(self, '_group'):
			from .Group import Group
			self._group = Group(self._core, self._cmd_group)
		return self._group

	def get_alias(self) -> str:
		"""SCPI: SCENario:CPDW:ALIas \n
		Snippet: value: str = driver.scenario.cpdw.get_alias() \n
		Enters an alias name. \n
			:return: alias: string
		"""
		response = self._core.io.query_str('SCENario:CPDW:ALIas?')
		return trim_str_response(response)

	def set_alias(self, alias: str) -> None:
		"""SCPI: SCENario:CPDW:ALIas \n
		Snippet: driver.scenario.cpdw.set_alias(alias = '1') \n
		Enters an alias name. \n
			:param alias: string
		"""
		param = Conversions.value_to_quoted_str(alias)
		self._core.io.write(f'SCENario:CPDW:ALIas {param}')

	def clear(self) -> None:
		"""SCPI: SCENario:CPDW:CLEar \n
		Snippet: driver.scenario.cpdw.clear() \n
		Deletes all items from the list or the table. \n
		"""
		self._core.io.write(f'SCENario:CPDW:CLEar')

	def clear_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: SCENario:CPDW:CLEar \n
		Snippet: driver.scenario.cpdw.clear_with_opc() \n
		Deletes all items from the list or the table. \n
		Same as clear, but waits for the operation to complete before continuing further. Use the RsPulseSeq.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'SCENario:CPDW:CLEar', opc_timeout_ms)

	def delete(self, delete: float) -> None:
		"""SCPI: SCENario:CPDW:DELete \n
		Snippet: driver.scenario.cpdw.delete(delete = 1.0) \n
		Deletes the particular item. \n
			:param delete: float
		"""
		param = Conversions.decimal_value_to_str(delete)
		self._core.io.write(f'SCENario:CPDW:DELete {param}')

	def get_enable(self) -> bool:
		"""SCPI: SCENario:CPDW:ENABle \n
		Snippet: value: bool = driver.scenario.cpdw.get_enable() \n
		If enabled, the PDW list is included in the output file. \n
			:return: enable: ON| OFF| 1| 0
		"""
		response = self._core.io.query_str('SCENario:CPDW:ENABle?')
		return Conversions.str_to_bool(response)

	def set_enable(self, enable: bool) -> None:
		"""SCPI: SCENario:CPDW:ENABle \n
		Snippet: driver.scenario.cpdw.set_enable(enable = False) \n
		If enabled, the PDW list is included in the output file. \n
			:param enable: ON| OFF| 1| 0
		"""
		param = Conversions.bool_to_str(enable)
		self._core.io.write(f'SCENario:CPDW:ENABle {param}')

	def get_freq(self) -> float:
		"""SCPI: SCENario:CPDW:FREQ \n
		Snippet: value: float = driver.scenario.cpdw.get_freq() \n
		Sets the frequency for the selected emitter. \n
			:return: freq: float Range: -1000 to 4.4e+10
		"""
		response = self._core.io.query_str('SCENario:CPDW:FREQ?')
		return Conversions.str_to_float(response)

	def set_freq(self, freq: float) -> None:
		"""SCPI: SCENario:CPDW:FREQ \n
		Snippet: driver.scenario.cpdw.set_freq(freq = 1.0) \n
		Sets the frequency for the selected emitter. \n
			:param freq: float Range: -1000 to 4.4e+10
		"""
		param = Conversions.decimal_value_to_str(freq)
		self._core.io.write(f'SCENario:CPDW:FREQ {param}')

	def get_interleaving(self) -> bool:
		"""SCPI: SCENario:CPDW:INTerleaving \n
		Snippet: value: bool = driver.scenario.cpdw.get_interleaving() \n
		If enabled, multiple PDW lists are interleaved into a single output file using a priority-based dropping algorithm. Set
		the priority with the command method RsPulseSeq.Scenario.Cpdw.priority. \n
			:return: interleaving: ON| OFF| 1| 0
		"""
		response = self._core.io.query_str('SCENario:CPDW:INTerleaving?')
		return Conversions.str_to_bool(response)

	def set_interleaving(self, interleaving: bool) -> None:
		"""SCPI: SCENario:CPDW:INTerleaving \n
		Snippet: driver.scenario.cpdw.set_interleaving(interleaving = False) \n
		If enabled, multiple PDW lists are interleaved into a single output file using a priority-based dropping algorithm. Set
		the priority with the command method RsPulseSeq.Scenario.Cpdw.priority. \n
			:param interleaving: ON| OFF| 1| 0
		"""
		param = Conversions.bool_to_str(interleaving)
		self._core.io.write(f'SCENario:CPDW:INTerleaving {param}')

	def get_ldelay(self) -> float:
		"""SCPI: SCENario:CPDW:LDELay \n
		Snippet: value: float = driver.scenario.cpdw.get_ldelay() \n
		If interleaving is enabled, shifts the processing of the selected PDW list in time. \n
			:return: ldelay: float Range: -1e+09 to 1e+09
		"""
		response = self._core.io.query_str('SCENario:CPDW:LDELay?')
		return Conversions.str_to_float(response)

	def set_ldelay(self, ldelay: float) -> None:
		"""SCPI: SCENario:CPDW:LDELay \n
		Snippet: driver.scenario.cpdw.set_ldelay(ldelay = 1.0) \n
		If interleaving is enabled, shifts the processing of the selected PDW list in time. \n
			:param ldelay: float Range: -1e+09 to 1e+09
		"""
		param = Conversions.decimal_value_to_str(ldelay)
		self._core.io.write(f'SCENario:CPDW:LDELay {param}')

	def get_level(self) -> float:
		"""SCPI: SCENario:CPDW:LEVel \n
		Snippet: value: float = driver.scenario.cpdw.get_level() \n
		Adds a level offset. \n
			:return: level: float Range: -200 to 0
		"""
		response = self._core.io.query_str('SCENario:CPDW:LEVel?')
		return Conversions.str_to_float(response)

	def set_level(self, level: float) -> None:
		"""SCPI: SCENario:CPDW:LEVel \n
		Snippet: driver.scenario.cpdw.set_level(level = 1.0) \n
		Adds a level offset. \n
			:param level: float Range: -200 to 0
		"""
		param = Conversions.decimal_value_to_str(level)
		self._core.io.write(f'SCENario:CPDW:LEVel {param}')

	def get_lvabs(self) -> float:
		"""SCPI: SCENario:CPDW:LVABs \n
		Snippet: value: float = driver.scenario.cpdw.get_lvabs() \n
		Sets the absolute level for the selected PDW list. \n
			:return: lvabs: float Range: -130 to 30
		"""
		response = self._core.io.query_str('SCENario:CPDW:LVABs?')
		return Conversions.str_to_float(response)

	def set_lvabs(self, lvabs: float) -> None:
		"""SCPI: SCENario:CPDW:LVABs \n
		Snippet: driver.scenario.cpdw.set_lvabs(lvabs = 1.0) \n
		Sets the absolute level for the selected PDW list. \n
			:param lvabs: float Range: -130 to 30
		"""
		param = Conversions.decimal_value_to_str(lvabs)
		self._core.io.write(f'SCENario:CPDW:LVABs {param}')

	def get_name(self) -> str:
		"""SCPI: SCENario:CPDW:NAME \n
		Snippet: value: str = driver.scenario.cpdw.get_name() \n
		Selects the waveform element, used to import the PDW list. Query the list of waveform elements with the command method
		RsPulseSeq.Waveform.catalog. \n
			:return: name: string
		"""
		response = self._core.io.query_str('SCENario:CPDW:NAME?')
		return trim_str_response(response)

	def set_name(self, name: str) -> None:
		"""SCPI: SCENario:CPDW:NAME \n
		Snippet: driver.scenario.cpdw.set_name(name = '1') \n
		Selects the waveform element, used to import the PDW list. Query the list of waveform elements with the command method
		RsPulseSeq.Waveform.catalog. \n
			:param name: string
		"""
		param = Conversions.value_to_quoted_str(name)
		self._core.io.write(f'SCENario:CPDW:NAME {param}')

	def get_priority(self) -> float:
		"""SCPI: SCENario:CPDW:PRIority \n
		Snippet: value: float = driver.scenario.cpdw.get_priority() \n
		Sets the priority of the selected PDW list , where the higher the value the higher the priority. \n
			:return: priority: float Range: 1 to 100
		"""
		response = self._core.io.query_str('SCENario:CPDW:PRIority?')
		return Conversions.str_to_float(response)

	def set_priority(self, priority: float) -> None:
		"""SCPI: SCENario:CPDW:PRIority \n
		Snippet: driver.scenario.cpdw.set_priority(priority = 1.0) \n
		Sets the priority of the selected PDW list , where the higher the value the higher the priority. \n
			:param priority: float Range: 1 to 100
		"""
		param = Conversions.decimal_value_to_str(priority)
		self._core.io.write(f'SCENario:CPDW:PRIority {param}')

	def get_select(self) -> float:
		"""SCPI: SCENario:CPDW:SELect \n
		Snippet: value: float = driver.scenario.cpdw.get_select() \n
		Selects the item to which the subsequent commands apply. \n
			:return: select: float Item number within the range 1 to ...:COUNt. For example, method RsPulseSeq.Sequence.Item.count. Range: 1 to 4096
		"""
		response = self._core.io.query_str('SCENario:CPDW:SELect?')
		return Conversions.str_to_float(response)

	def set_select(self, select: float) -> None:
		"""SCPI: SCENario:CPDW:SELect \n
		Snippet: driver.scenario.cpdw.set_select(select = 1.0) \n
		Selects the item to which the subsequent commands apply. \n
			:param select: float Item number within the range 1 to ...:COUNt. For example, method RsPulseSeq.Sequence.Item.count. Range: 1 to 4096
		"""
		param = Conversions.decimal_value_to_str(select)
		self._core.io.write(f'SCENario:CPDW:SELect {param}')

	def get_threshold(self) -> float:
		"""SCPI: SCENario:CPDW:THReshold \n
		Snippet: value: float = driver.scenario.cpdw.get_threshold() \n
		Sets a threshold. Pulses at levels below this threshold are omitted. \n
			:return: threshold: float Range: -100 to 0
		"""
		response = self._core.io.query_str('SCENario:CPDW:THReshold?')
		return Conversions.str_to_float(response)

	def set_threshold(self, threshold: float) -> None:
		"""SCPI: SCENario:CPDW:THReshold \n
		Snippet: driver.scenario.cpdw.set_threshold(threshold = 1.0) \n
		Sets a threshold. Pulses at levels below this threshold are omitted. \n
			:param threshold: float Range: -100 to 0
		"""
		param = Conversions.decimal_value_to_str(threshold)
		self._core.io.write(f'SCENario:CPDW:THReshold {param}')

	def clone(self) -> 'Cpdw':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = Cpdw(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Mode:
	"""Mode commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("mode", core, parent)

	def get_beam(self) -> float:
		"""SCPI: SCENario:EMITter:MODE:BEAM \n
		Snippet: value: float = driver.scenario.emitter.mode.get_beam() \n
		Sets the used beam of the current mode. \n
			:return: beam: float Range: 1 to 32
		"""
		response = self._core.io.query_str('SCENario:EMITter:MODE:BEAM?')
		return Conversions.str_to_float(response)

	def set_beam(self, beam: float) -> None:
		"""SCPI: SCENario:EMITter:MODE:BEAM \n
		Snippet: driver.scenario.emitter.mode.set_beam(beam = 1.0) \n
		Sets the used beam of the current mode. \n
			:param beam: float Range: 1 to 32
		"""
		param = Conversions.decimal_value_to_str(beam)
		self._core.io.write(f'SCENario:EMITter:MODE:BEAM {param}')

	def get_value(self) -> float:
		"""SCPI: SCENario:EMITter:MODE \n
		Snippet: value: float = driver.scenario.emitter.mode.get_value() \n
		Set the emitter mode. \n
			:return: mode: float Range: 1 to 32
		"""
		response = self._core.io.query_str('SCENario:EMITter:MODE?')
		return Conversions.str_to_float(response)

	def set_value(self, mode: float) -> None:
		"""SCPI: SCENario:EMITter:MODE \n
		Snippet: driver.scenario.emitter.mode.set_value(mode = 1.0) \n
		Set the emitter mode. \n
			:param mode: float Range: 1 to 32
		"""
		param = Conversions.decimal_value_to_str(mode)
		self._core.io.write(f'SCENario:EMITter:MODE {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Init:
	"""Init commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("init", core, parent)

	def get_float_value(self) -> float:
		"""SCPI: DSRC:ITEM:PRBS:INIT:VALue \n
		Snippet: value: float = driver.dsrc.item.prbs.init.get_float_value() \n
		Set a new initialization value. \n
			:return: value: float Range: 1 to 511
		"""
		response = self._core.io.query_str('DSRC:ITEM:PRBS:INIT:VALue?')
		return Conversions.str_to_float(response)

	def set_float_value(self, value: float) -> None:
		"""SCPI: DSRC:ITEM:PRBS:INIT:VALue \n
		Snippet: driver.dsrc.item.prbs.init.set_float_value(value = 1.0) \n
		Set a new initialization value. \n
			:param value: float Range: 1 to 511
		"""
		param = Conversions.decimal_value_to_str(value)
		self._core.io.write(f'DSRC:ITEM:PRBS:INIT:VALue {param}')

	def get_value(self) -> bool:
		"""SCPI: DSRC:ITEM:PRBS:INIT \n
		Snippet: value: bool = driver.dsrc.item.prbs.init.get_value() \n
		Enables/disables initialization of the shift register with a user-defined value. \n
			:return: init: ON| OFF
		"""
		response = self._core.io.query_str('DSRC:ITEM:PRBS:INIT?')
		return Conversions.str_to_bool(response)

	def set_value(self, init: bool) -> None:
		"""SCPI: DSRC:ITEM:PRBS:INIT \n
		Snippet: driver.dsrc.item.prbs.init.set_value(init = False) \n
		Enables/disables initialization of the shift register with a user-defined value. \n
			:param init: ON| OFF
		"""
		param = Conversions.bool_to_str(init)
		self._core.io.write(f'DSRC:ITEM:PRBS:INIT {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from ....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Emitter:
	"""Emitter commands group definition. 24 total commands, 3 Subgroups, 15 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("emitter", core, parent)

	@property
	def add(self):
		"""add commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_add'):
			from .Add import Add
			self._add = Add(self._core, self._cmd_group)
		return self._add

	@property
	def blankRanges(self):
		"""blankRanges commands group. 1 Sub-classes, 6 commands."""
		if not hasattr(self, '_blankRanges'):
			from .BlankRanges import BlankRanges
			self._blankRanges = BlankRanges(self._core, self._cmd_group)
		return self._blankRanges

	@property
	def direction(self):
		"""direction commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_direction'):
			from .Direction import Direction
			self._direction = Direction(self._core, self._cmd_group)
		return self._direction

	def get_alias(self) -> str:
		"""SCPI: PLATform:EMITter:ALIas \n
		Snippet: value: str = driver.platform.emitter.get_alias() \n
		Sets an alias name for the selected platform emitter element. \n
			:return: alias: string
		"""
		response = self._core.io.query_str('PLATform:EMITter:ALIas?')
		return trim_str_response(response)

	def set_alias(self, alias: str) -> None:
		"""SCPI: PLATform:EMITter:ALIas \n
		Snippet: driver.platform.emitter.set_alias(alias = '1') \n
		Sets an alias name for the selected platform emitter element. \n
			:param alias: string
		"""
		param = Conversions.value_to_quoted_str(alias)
		self._core.io.write(f'PLATform:EMITter:ALIas {param}')

	def get_angle(self) -> float:
		"""SCPI: PLATform:EMITter:ANGLe \n
		Snippet: value: float = driver.platform.emitter.get_angle() \n
		You can set the position of the selected emitter relative to the platform's origin, using this command combined with
		method RsPulseSeq.Platform.Emitter.radius.
			- method RsPulseSeq.Platform.Emitter.angle sets the angle of the emitter element on the azimuth plane, relative to the platform's heading.
			- method RsPulseSeq.Platform.Emitter.radius sets the distance of the emitter element on the azimuth plane, relative to the platform's origin. \n
			:return: angle: float Range: 0 to 360
		"""
		response = self._core.io.query_str('PLATform:EMITter:ANGLe?')
		return Conversions.str_to_float(response)

	def set_angle(self, angle: float) -> None:
		"""SCPI: PLATform:EMITter:ANGLe \n
		Snippet: driver.platform.emitter.set_angle(angle = 1.0) \n
		You can set the position of the selected emitter relative to the platform's origin, using this command combined with
		method RsPulseSeq.Platform.Emitter.radius.
			- method RsPulseSeq.Platform.Emitter.angle sets the angle of the emitter element on the azimuth plane, relative to the platform's heading.
			- method RsPulseSeq.Platform.Emitter.radius sets the distance of the emitter element on the azimuth plane, relative to the platform's origin. \n
			:param angle: float Range: 0 to 360
		"""
		param = Conversions.decimal_value_to_str(angle)
		self._core.io.write(f'PLATform:EMITter:ANGLe {param}')

	def get_azimuth(self) -> float:
		"""SCPI: PLATform:EMITter:AZIMuth \n
		Snippet: value: float = driver.platform.emitter.get_azimuth() \n
		Angle of the emitter element's pointing direction relative to the platform's heading. \n
			:return: azimuth: float Range: 0 to 360
		"""
		response = self._core.io.query_str('PLATform:EMITter:AZIMuth?')
		return Conversions.str_to_float(response)

	def set_azimuth(self, azimuth: float) -> None:
		"""SCPI: PLATform:EMITter:AZIMuth \n
		Snippet: driver.platform.emitter.set_azimuth(azimuth = 1.0) \n
		Angle of the emitter element's pointing direction relative to the platform's heading. \n
			:param azimuth: float Range: 0 to 360
		"""
		param = Conversions.decimal_value_to_str(azimuth)
		self._core.io.write(f'PLATform:EMITter:AZIMuth {param}')

	def get_bm(self) -> str:
		"""SCPI: PLATform:EMITter:BM \n
		Snippet: value: str = driver.platform.emitter.get_bm() \n
		No command help available \n
			:return: bm: No help available
		"""
		response = self._core.io.query_str('PLATform:EMITter:BM?')
		return trim_str_response(response)

	def set_bm(self, bm: str) -> None:
		"""SCPI: PLATform:EMITter:BM \n
		Snippet: driver.platform.emitter.set_bm(bm = '1') \n
		No command help available \n
			:param bm: No help available
		"""
		param = Conversions.value_to_quoted_str(bm)
		self._core.io.write(f'PLATform:EMITter:BM {param}')

	def get_bmid(self) -> str:
		"""SCPI: PLATform:EMITter:BMID \n
		Snippet: value: str = driver.platform.emitter.get_bmid() \n
		No command help available \n
			:return: bmid: No help available
		"""
		response = self._core.io.query_str('PLATform:EMITter:BMID?')
		return trim_str_response(response)

	def set_bmid(self, bmid: str) -> None:
		"""SCPI: PLATform:EMITter:BMID \n
		Snippet: driver.platform.emitter.set_bmid(bmid = '1') \n
		No command help available \n
			:param bmid: No help available
		"""
		param = Conversions.value_to_quoted_str(bmid)
		self._core.io.write(f'PLATform:EMITter:BMID {param}')

	def clear(self) -> None:
		"""SCPI: PLATform:EMITter:CLEar \n
		Snippet: driver.platform.emitter.clear() \n
		Deletes all items from the list or the table. \n
		"""
		self._core.io.write(f'PLATform:EMITter:CLEar')

	def clear_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: PLATform:EMITter:CLEar \n
		Snippet: driver.platform.emitter.clear_with_opc() \n
		Deletes all items from the list or the table. \n
		Same as clear, but waits for the operation to complete before continuing further. Use the RsPulseSeq.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'PLATform:EMITter:CLEar', opc_timeout_ms)

	def delete(self, delete: float) -> None:
		"""SCPI: PLATform:EMITter:DELete \n
		Snippet: driver.platform.emitter.delete(delete = 1.0) \n
		Deletes the particular item. \n
			:param delete: float
		"""
		param = Conversions.decimal_value_to_str(delete)
		self._core.io.write(f'PLATform:EMITter:DELete {param}')

	def get_elevation(self) -> float:
		"""SCPI: PLATform:EMITter:ELEVation \n
		Snippet: value: float = driver.platform.emitter.get_elevation() \n
		Elevation of the emitter item's pointing direction, relative to the azimuth plane. \n
			:return: elevation: float Range: -90 to 90
		"""
		response = self._core.io.query_str('PLATform:EMITter:ELEVation?')
		return Conversions.str_to_float(response)

	def set_elevation(self, elevation: float) -> None:
		"""SCPI: PLATform:EMITter:ELEVation \n
		Snippet: driver.platform.emitter.set_elevation(elevation = 1.0) \n
		Elevation of the emitter item's pointing direction, relative to the azimuth plane. \n
			:param elevation: float Range: -90 to 90
		"""
		param = Conversions.decimal_value_to_str(elevation)
		self._core.io.write(f'PLATform:EMITter:ELEVation {param}')

	def get_height(self) -> float:
		"""SCPI: PLATform:EMITter:HEIGht \n
		Snippet: value: float = driver.platform.emitter.get_height() \n
		Height of the selected emitter element relative to the platform's origin. Can be used, for example, to differentiate
		between:
			- Radars mounted on different parts of a ship or aircraft.
			- Various radars situated across a land-based radar installation. \n
			:return: height: float Range: -500 to 500
		"""
		response = self._core.io.query_str('PLATform:EMITter:HEIGht?')
		return Conversions.str_to_float(response)

	def set_height(self, height: float) -> None:
		"""SCPI: PLATform:EMITter:HEIGht \n
		Snippet: driver.platform.emitter.set_height(height = 1.0) \n
		Height of the selected emitter element relative to the platform's origin. Can be used, for example, to differentiate
		between:
			- Radars mounted on different parts of a ship or aircraft.
			- Various radars situated across a land-based radar installation. \n
			:param height: float Range: -500 to 500
		"""
		param = Conversions.decimal_value_to_str(height)
		self._core.io.write(f'PLATform:EMITter:HEIGht {param}')

	def get_radius(self) -> float:
		"""SCPI: PLATform:EMITter:RADius \n
		Snippet: value: float = driver.platform.emitter.get_radius() \n
		You can set the position of the selected emitter relative to the platform's origin, using this command combined with
		method RsPulseSeq.Platform.Emitter.angle.
			- method RsPulseSeq.Platform.Emitter.angle sets the angle of the emitter element on the azimuth plane, relative to the platform's heading.
			- method RsPulseSeq.Platform.Emitter.radius sets the distance of the emitter element on the azimuth plane, relative to the platform's origin. \n
			:return: radius: float Range: 0 to 2000
		"""
		response = self._core.io.query_str('PLATform:EMITter:RADius?')
		return Conversions.str_to_float(response)

	def set_radius(self, radius: float) -> None:
		"""SCPI: PLATform:EMITter:RADius \n
		Snippet: driver.platform.emitter.set_radius(radius = 1.0) \n
		You can set the position of the selected emitter relative to the platform's origin, using this command combined with
		method RsPulseSeq.Platform.Emitter.angle.
			- method RsPulseSeq.Platform.Emitter.angle sets the angle of the emitter element on the azimuth plane, relative to the platform's heading.
			- method RsPulseSeq.Platform.Emitter.radius sets the distance of the emitter element on the azimuth plane, relative to the platform's origin. \n
			:param radius: float Range: 0 to 2000
		"""
		param = Conversions.decimal_value_to_str(radius)
		self._core.io.write(f'PLATform:EMITter:RADius {param}')

	def get_roll(self) -> float:
		"""SCPI: PLATform:EMITter:ROLL \n
		Snippet: value: float = driver.platform.emitter.get_roll() \n
		Roll of the emitter item's pointing direction relative to the platform's up direction. Can be used, for example, to
		simulate the emissions from a mast-mounted radar on a marine platform affected by wind. \n
			:return: roll: float Range: -180 to 180
		"""
		response = self._core.io.query_str('PLATform:EMITter:ROLL?')
		return Conversions.str_to_float(response)

	def set_roll(self, roll: float) -> None:
		"""SCPI: PLATform:EMITter:ROLL \n
		Snippet: driver.platform.emitter.set_roll(roll = 1.0) \n
		Roll of the emitter item's pointing direction relative to the platform's up direction. Can be used, for example, to
		simulate the emissions from a mast-mounted radar on a marine platform affected by wind. \n
			:param roll: float Range: -180 to 180
		"""
		param = Conversions.decimal_value_to_str(roll)
		self._core.io.write(f'PLATform:EMITter:ROLL {param}')

	def get_select(self) -> float:
		"""SCPI: PLATform:EMITter:SELect \n
		Snippet: value: float = driver.platform.emitter.get_select() \n
		Selects the repository element to which the subsequent commands apply. \n
			:return: select: string Element name, as defined with the ...:CREate or ...:NAME command. To query the existing elements, use the ...:CATalog? command. For example, method RsPulseSeq.Repository.catalog.
		"""
		response = self._core.io.query_str('PLATform:EMITter:SELect?')
		return Conversions.str_to_float(response)

	def set_select(self, select: float) -> None:
		"""SCPI: PLATform:EMITter:SELect \n
		Snippet: driver.platform.emitter.set_select(select = 1.0) \n
		Selects the repository element to which the subsequent commands apply. \n
			:param select: string Element name, as defined with the ...:CREate or ...:NAME command. To query the existing elements, use the ...:CATalog? command. For example, method RsPulseSeq.Repository.catalog.
		"""
		param = Conversions.decimal_value_to_str(select)
		self._core.io.write(f'PLATform:EMITter:SELect {param}')

	def get_x(self) -> float:
		"""SCPI: PLATform:EMITter:X \n
		Snippet: value: float = driver.platform.emitter.get_x() \n
		Set the position of the selected emitter relative to the platform's origin, using this command combined with method
		RsPulseSeq.Platform.Emitter.y.
			INTRO_CMD_HELP: X and Y represent the two principle axis of the platform. \n
			- The Y-axis represents the axis along the center-line of the platform. This axis:
			Table Header:  \n
			- Corresponds to its heading.
			- Passes through the origin.
			- The X-axis:
			Table Header:  \n
			- Is at right-angles to the Y-axis.
			- Passes through the origin.
			INTRO_CMD_HELP: X and Y represent the two principle axis of the platform. \n
			- method RsPulseSeq.Platform.Emitter.y sets the distance of the emitter element from the origin, along the Y-axis. Positive values are towards the heading.
			- method RsPulseSeq.Platform.Emitter.x sets the distance of the emitter element from the origin, along the X-axis. \n
			:return: x: float Range: -2000 to 2000
		"""
		response = self._core.io.query_str('PLATform:EMITter:X?')
		return Conversions.str_to_float(response)

	def set_x(self, x: float) -> None:
		"""SCPI: PLATform:EMITter:X \n
		Snippet: driver.platform.emitter.set_x(x = 1.0) \n
		Set the position of the selected emitter relative to the platform's origin, using this command combined with method
		RsPulseSeq.Platform.Emitter.y.
			INTRO_CMD_HELP: X and Y represent the two principle axis of the platform. \n
			- The Y-axis represents the axis along the center-line of the platform. This axis:
			Table Header:  \n
			- Corresponds to its heading.
			- Passes through the origin.
			- The X-axis:
			Table Header:  \n
			- Is at right-angles to the Y-axis.
			- Passes through the origin.
			INTRO_CMD_HELP: X and Y represent the two principle axis of the platform. \n
			- method RsPulseSeq.Platform.Emitter.y sets the distance of the emitter element from the origin, along the Y-axis. Positive values are towards the heading.
			- method RsPulseSeq.Platform.Emitter.x sets the distance of the emitter element from the origin, along the X-axis. \n
			:param x: float Range: -2000 to 2000
		"""
		param = Conversions.decimal_value_to_str(x)
		self._core.io.write(f'PLATform:EMITter:X {param}')

	def get_y(self) -> float:
		"""SCPI: PLATform:EMITter:Y \n
		Snippet: value: float = driver.platform.emitter.get_y() \n
		Set the position of the selected emitter relative to the platform's origin, using this command combined with method
		RsPulseSeq.Platform.Emitter.x.
			INTRO_CMD_HELP: X and Y represent the two principle axis of the platform. \n
			- The Y-axis represents the axis along the center-line of the platform. This axis:
			Table Header:  \n
			- Corresponds to its heading.
			- Passes through the origin.
			- The X-axis:
			Table Header:  \n
			- Is at right-angles to the Y-axis.
			- Passes through the origin.
			INTRO_CMD_HELP: X and Y represent the two principle axis of the platform. \n
			- method RsPulseSeq.Platform.Emitter.y sets the distance of the emitter element from the origin, along the Y-axis. Positive values are towards the heading. Step = 0.01 m
			- method RsPulseSeq.Platform.Emitter.x sets the distance of the emitter element from the origin, along the X-axis. Step = 0.01 m \n
			:return: y: float Range: -2000 to 2000
		"""
		response = self._core.io.query_str('PLATform:EMITter:Y?')
		return Conversions.str_to_float(response)

	def set_y(self, y: float) -> None:
		"""SCPI: PLATform:EMITter:Y \n
		Snippet: driver.platform.emitter.set_y(y = 1.0) \n
		Set the position of the selected emitter relative to the platform's origin, using this command combined with method
		RsPulseSeq.Platform.Emitter.x.
			INTRO_CMD_HELP: X and Y represent the two principle axis of the platform. \n
			- The Y-axis represents the axis along the center-line of the platform. This axis:
			Table Header:  \n
			- Corresponds to its heading.
			- Passes through the origin.
			- The X-axis:
			Table Header:  \n
			- Is at right-angles to the Y-axis.
			- Passes through the origin.
			INTRO_CMD_HELP: X and Y represent the two principle axis of the platform. \n
			- method RsPulseSeq.Platform.Emitter.y sets the distance of the emitter element from the origin, along the Y-axis. Positive values are towards the heading. Step = 0.01 m
			- method RsPulseSeq.Platform.Emitter.x sets the distance of the emitter element from the origin, along the X-axis. Step = 0.01 m \n
			:param y: float Range: -2000 to 2000
		"""
		param = Conversions.decimal_value_to_str(y)
		self._core.io.write(f'PLATform:EMITter:Y {param}')

	def get_value(self) -> str:
		"""SCPI: PLATform:EMITter \n
		Snippet: value: str = driver.platform.emitter.get_value() \n
		The string must be unique within the repository. Letters, numbers, spaces and some special characters can be used.
			INTRO_CMD_HELP: Examples of special characters: \n
			- Supported: !$% =?-+_.
			- Not supported: §&/:äöü \n
			:return: emitter: string
		"""
		response = self._core.io.query_str('PLATform:EMITter?')
		return trim_str_response(response)

	def set_value(self, emitter: str) -> None:
		"""SCPI: PLATform:EMITter \n
		Snippet: driver.platform.emitter.set_value(emitter = '1') \n
		The string must be unique within the repository. Letters, numbers, spaces and some special characters can be used.
			INTRO_CMD_HELP: Examples of special characters: \n
			- Supported: !$% =?-+_.
			- Not supported: §&/:äöü \n
			:param emitter: string
		"""
		param = Conversions.value_to_quoted_str(emitter)
		self._core.io.write(f'PLATform:EMITter {param}')

	def clone(self) -> 'Emitter':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = Emitter(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class U:
	"""U commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("u", core, parent)

	def get_center(self) -> float:
		"""SCPI: IPM:RANDom:U:CENTer \n
		Snippet: value: float = driver.ipm.random.u.get_center() \n
		Sets the center parameter of the U distribution. \n
			:return: center: float Range: -1e+09 to 1e+09
		"""
		response = self._core.io.query_str('IPM:RANDom:U:CENTer?')
		return Conversions.str_to_float(response)

	def set_center(self, center: float) -> None:
		"""SCPI: IPM:RANDom:U:CENTer \n
		Snippet: driver.ipm.random.u.set_center(center = 1.0) \n
		Sets the center parameter of the U distribution. \n
			:param center: float Range: -1e+09 to 1e+09
		"""
		param = Conversions.decimal_value_to_str(center)
		self._core.io.write(f'IPM:RANDom:U:CENTer {param}')

	def get_range(self) -> float:
		"""SCPI: IPM:RANDom:U:RANGe \n
		Snippet: value: float = driver.ipm.random.u.get_range() \n
		Sets the range parameter of the U distribution. \n
			:return: range_py: float Range: 1e-09 to 1e+09
		"""
		response = self._core.io.query_str('IPM:RANDom:U:RANGe?')
		return Conversions.str_to_float(response)

	def set_range(self, range_py: float) -> None:
		"""SCPI: IPM:RANDom:U:RANGe \n
		Snippet: driver.ipm.random.u.set_range(range_py = 1.0) \n
		Sets the range parameter of the U distribution. \n
			:param range_py: float Range: 1e-09 to 1e+09
		"""
		param = Conversions.decimal_value_to_str(range_py)
		self._core.io.write(f'IPM:RANDom:U:RANGe {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from ....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Firing:
	"""Firing commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("firing", core, parent)

	def get_enable(self) -> bool:
		"""SCPI: IPM:LIST:FIRing:ENABle \n
		Snippet: value: bool = driver.ipm.listPy.firing.get_enable() \n
		Enables using firing order for list-based IPM profiles. \n
			:return: enable: ON| OFF| 1| 0
		"""
		response = self._core.io.query_str('IPM:LIST:FIRing:ENABle?')
		return Conversions.str_to_bool(response)

	def set_enable(self, enable: bool) -> None:
		"""SCPI: IPM:LIST:FIRing:ENABle \n
		Snippet: driver.ipm.listPy.firing.set_enable(enable = False) \n
		Enables using firing order for list-based IPM profiles. \n
			:param enable: ON| OFF| 1| 0
		"""
		param = Conversions.bool_to_str(enable)
		self._core.io.write(f'IPM:LIST:FIRing:ENABle {param}')

	def get_sequence(self) -> str:
		"""SCPI: IPM:LIST:FIRing:SEQuence \n
		Snippet: value: str = driver.ipm.listPy.firing.get_sequence() \n
		Sets the firing order sequence. \n
			:return: sequence: string
		"""
		response = self._core.io.query_str('IPM:LIST:FIRing:SEQuence?')
		return trim_str_response(response)

	def set_sequence(self, sequence: str) -> None:
		"""SCPI: IPM:LIST:FIRing:SEQuence \n
		Snippet: driver.ipm.listPy.firing.set_sequence(sequence = '1') \n
		Sets the firing order sequence. \n
			:param sequence: string
		"""
		param = Conversions.value_to_quoted_str(sequence)
		self._core.io.write(f'IPM:LIST:FIRing:SEQuence {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Hexagonal:
	"""Hexagonal commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("hexagonal", core, parent)

	def get_distance(self) -> float:
		"""SCPI: ANTenna:MODel:CARRay:HEXagonal:DISTance \n
		Snippet: value: float = driver.antenna.model.carray.hexagonal.get_distance() \n
		Sets the spacing between the elements of the array antenna. \n
			:return: distance: No help available
		"""
		response = self._core.io.query_str('ANTenna:MODel:CARRay:HEXagonal:DISTance?')
		return Conversions.str_to_float(response)

	def set_distance(self, distance: float) -> None:
		"""SCPI: ANTenna:MODel:CARRay:HEXagonal:DISTance \n
		Snippet: driver.antenna.model.carray.hexagonal.set_distance(distance = 1.0) \n
		Sets the spacing between the elements of the array antenna. \n
			:param distance: float Range: 0.0001 to 1
		"""
		param = Conversions.decimal_value_to_str(distance)
		self._core.io.write(f'ANTenna:MODel:CARRay:HEXagonal:DISTance {param}')

	def get_n(self) -> float:
		"""SCPI: ANTenna:MODel:CARRay:HEXagonal:N \n
		Snippet: value: float = driver.antenna.model.carray.hexagonal.get_n() \n
		Sets the number of elements of the antenna array. \n
			:return: n: No help available
		"""
		response = self._core.io.query_str('ANTenna:MODel:CARRay:HEXagonal:N?')
		return Conversions.str_to_float(response)

	def set_n(self, n: float) -> None:
		"""SCPI: ANTenna:MODel:CARRay:HEXagonal:N \n
		Snippet: driver.antenna.model.carray.hexagonal.set_n(n = 1.0) \n
		Sets the number of elements of the antenna array. \n
			:param n: float Range: 2 to 1000 (planar phased array; linear phase array) , 100 (rectangular phase array) , 50 (hexagonal phase array)
		"""
		param = Conversions.decimal_value_to_str(n)
		self._core.io.write(f'ANTenna:MODel:CARRay:HEXagonal:N {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ...Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Path:
	"""Path commands group definition. 3 total commands, 0 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("path", core, parent)

	def set_add(self, add: str) -> None:
		"""SCPI: REPManager:PATH:ADD \n
		Snippet: driver.repmanager.path.set_add(add = '1') \n
		Add the selected directory. \n
			:param add: string Complete file path
		"""
		param = Conversions.value_to_quoted_str(add)
		self._core.io.write(f'REPManager:PATH:ADD {param}')

	def delete(self, delete: str) -> None:
		"""SCPI: REPManager:PATH:DELete \n
		Snippet: driver.repmanager.path.delete(delete = '1') \n
		Removes the selected file path. \n
			:param delete: string File path
		"""
		param = Conversions.value_to_quoted_str(delete)
		self._core.io.write(f'REPManager:PATH:DELete {param}')

	def get_list_py(self) -> str:
		"""SCPI: REPManager:PATH:LIST \n
		Snippet: value: str = driver.repmanager.path.get_list_py() \n
		Queries the directory in that the repository files are stored. \n
			:return: list_py: string Compete file path
		"""
		response = self._core.io.query_str('REPManager:PATH:LIST?')
		return trim_str_response(response)



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Subitem:
	"""Subitem commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("subitem", core, parent)

	def get_current(self) -> float:
		"""SCPI: SCENario:DF:SUBitem:CURRent \n
		Snippet: value: float = driver.scenario.df.subitem.get_current() \n
		No command help available \n
			:return: current: float Range: 1 to 4096
		"""
		response = self._core.io.query_str('SCENario:DF:SUBitem:CURRent?')
		return Conversions.str_to_float(response)

	def set_current(self, current: float) -> None:
		"""SCPI: SCENario:DF:SUBitem:CURRent \n
		Snippet: driver.scenario.df.subitem.set_current(current = 1.0) \n
		No command help available \n
			:param current: float Range: 1 to 4096
		"""
		param = Conversions.decimal_value_to_str(current)
		self._core.io.write(f'SCENario:DF:SUBitem:CURRent {param}')

	def get_select(self) -> float:
		"""SCPI: SCENario:DF:SUBitem:SELect \n
		Snippet: value: float = driver.scenario.df.subitem.get_select() \n
		No command help available \n
			:return: select: float Range: 1 to 4096
		"""
		response = self._core.io.query_str('SCENario:DF:SUBitem:SELect?')
		return Conversions.str_to_float(response)

	def set_select(self, select: float) -> None:
		"""SCPI: SCENario:DF:SUBitem:SELect \n
		Snippet: driver.scenario.df.subitem.set_select(select = 1.0) \n
		No command help available \n
			:param select: float Range: 1 to 4096
		"""
		param = Conversions.decimal_value_to_str(select)
		self._core.io.write(f'SCENario:DF:SUBitem:SELect {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Lsw:
	"""Lsw commands group definition. 5 total commands, 0 Subgroups, 5 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("lsw", core, parent)

	# noinspection PyTypeChecker
	def get_direction(self) -> enums.LswDirection:
		"""SCPI: SCAN:LSW:DIRection \n
		Snippet: value: enums.LswDirection = driver.scan.lsw.get_direction() \n
		Sets the horizontal or vertical switching direction. \n
			:return: direction: H| V
		"""
		response = self._core.io.query_str('SCAN:LSW:DIRection?')
		return Conversions.str_to_scalar_enum(response, enums.LswDirection)

	def set_direction(self, direction: enums.LswDirection) -> None:
		"""SCPI: SCAN:LSW:DIRection \n
		Snippet: driver.scan.lsw.set_direction(direction = enums.LswDirection.H) \n
		Sets the horizontal or vertical switching direction. \n
			:param direction: H| V
		"""
		param = Conversions.enum_scalar_to_str(direction, enums.LswDirection)
		self._core.io.write(f'SCAN:LSW:DIRection {param}')

	def get_dwell(self) -> float:
		"""SCPI: SCAN:LSW:DWELl \n
		Snippet: value: float = driver.scan.lsw.get_dwell() \n
		Sets the speed with that the antenna switches between the lobes. \n
			:return: dwell: float Range: 1e-06 to 1
		"""
		response = self._core.io.query_str('SCAN:LSW:DWELl?')
		return Conversions.str_to_float(response)

	def set_dwell(self, dwell: float) -> None:
		"""SCPI: SCAN:LSW:DWELl \n
		Snippet: driver.scan.lsw.set_dwell(dwell = 1.0) \n
		Sets the speed with that the antenna switches between the lobes. \n
			:param dwell: float Range: 1e-06 to 1
		"""
		param = Conversions.decimal_value_to_str(dwell)
		self._core.io.write(f'SCAN:LSW:DWELl {param}')

	# noinspection PyTypeChecker
	def get_lobes(self) -> enums.LobesCount:
		"""SCPI: SCAN:LSW:LOBes \n
		Snippet: value: enums.LobesCount = driver.scan.lsw.get_lobes() \n
		Set the number of lobes. \n
			:return: lobes: 2| 4
		"""
		response = self._core.io.query_str('SCAN:LSW:LOBes?')
		return Conversions.str_to_scalar_enum(response, enums.LobesCount)

	def set_lobes(self, lobes: enums.LobesCount) -> None:
		"""SCPI: SCAN:LSW:LOBes \n
		Snippet: driver.scan.lsw.set_lobes(lobes = enums.LobesCount._2) \n
		Set the number of lobes. \n
			:param lobes: 2| 4
		"""
		param = Conversions.enum_scalar_to_str(lobes, enums.LobesCount)
		self._core.io.write(f'SCAN:LSW:LOBes {param}')

	# noinspection PyTypeChecker
	def get_rotation(self) -> enums.Rotation:
		"""SCPI: SCAN:LSW:ROTation \n
		Snippet: value: enums.Rotation = driver.scan.lsw.get_rotation() \n
		Sets the rotation direction of the antenna. \n
			:return: rotation: CW| CCW
		"""
		response = self._core.io.query_str('SCAN:LSW:ROTation?')
		return Conversions.str_to_scalar_enum(response, enums.Rotation)

	def set_rotation(self, rotation: enums.Rotation) -> None:
		"""SCPI: SCAN:LSW:ROTation \n
		Snippet: driver.scan.lsw.set_rotation(rotation = enums.Rotation.CCW) \n
		Sets the rotation direction of the antenna. \n
			:param rotation: CW| CCW
		"""
		param = Conversions.enum_scalar_to_str(rotation, enums.Rotation)
		self._core.io.write(f'SCAN:LSW:ROTation {param}')

	def get_squint(self) -> float:
		"""SCPI: SCAN:LSW:SQUint \n
		Snippet: value: float = driver.scan.lsw.get_squint() \n
		Sets the offset angle of the antenna beam, that means for the conical antenna the parameter sets the radius of the
		scanned circle. \n
			:return: squint: float Range: 0.05 to 15, Unit: degree
		"""
		response = self._core.io.query_str('SCAN:LSW:SQUint?')
		return Conversions.str_to_float(response)

	def set_squint(self, squint: float) -> None:
		"""SCPI: SCAN:LSW:SQUint \n
		Snippet: driver.scan.lsw.set_squint(squint = 1.0) \n
		Sets the offset angle of the antenna beam, that means for the conical antenna the parameter sets the radius of the
		scanned circle. \n
			:param squint: float Range: 0.05 to 15, Unit: degree
		"""
		param = Conversions.decimal_value_to_str(squint)
		self._core.io.write(f'SCAN:LSW:SQUint {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class Array:
	"""Array commands group definition. 8 total commands, 1 Subgroups, 7 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("array", core, parent)

	@property
	def element(self):
		"""element commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_element'):
			from .Element import Element
			self._element = Element(self._core, self._cmd_group)
		return self._element

	def get_cosn(self) -> float:
		"""SCPI: ANTenna:MODel:ARRay:COSN \n
		Snippet: value: float = driver.antenna.model.array.get_cosn() \n
		Sets Cos^N of the Planar Phased Array antenna. \n
			:return: cosn: float Range: 2 to 10
		"""
		response = self._core.io.query_str('ANTenna:MODel:ARRay:COSN?')
		return Conversions.str_to_float(response)

	def set_cosn(self, cosn: float) -> None:
		"""SCPI: ANTenna:MODel:ARRay:COSN \n
		Snippet: driver.antenna.model.array.set_cosn(cosn = 1.0) \n
		Sets Cos^N of the Planar Phased Array antenna. \n
			:param cosn: float Range: 2 to 10
		"""
		param = Conversions.decimal_value_to_str(cosn)
		self._core.io.write(f'ANTenna:MODel:ARRay:COSN {param}')

	# noinspection PyTypeChecker
	def get_distribution(self) -> enums.AntennaModelArray:
		"""SCPI: ANTenna:MODel:ARRay:DISTribution \n
		Snippet: value: enums.AntennaModelArray = driver.antenna.model.array.get_distribution() \n
		Sets the aperture distribution of the Planar Phased Array antenna. \n
			:return: distribution: UNIForm| PARabolic| COSine| CSQuared| COSN| TRIangular| HAMMing| HANN
		"""
		response = self._core.io.query_str('ANTenna:MODel:ARRay:DISTribution?')
		return Conversions.str_to_scalar_enum(response, enums.AntennaModelArray)

	def set_distribution(self, distribution: enums.AntennaModelArray) -> None:
		"""SCPI: ANTenna:MODel:ARRay:DISTribution \n
		Snippet: driver.antenna.model.array.set_distribution(distribution = enums.AntennaModelArray.COSine) \n
		Sets the aperture distribution of the Planar Phased Array antenna. \n
			:param distribution: UNIForm| PARabolic| COSine| CSQuared| COSN| TRIangular| HAMMing| HANN
		"""
		param = Conversions.enum_scalar_to_str(distribution, enums.AntennaModelArray)
		self._core.io.write(f'ANTenna:MODel:ARRay:DISTribution {param}')

	def get_nx(self) -> float:
		"""SCPI: ANTenna:MODel:ARRay:NX \n
		Snippet: value: float = driver.antenna.model.array.get_nx() \n
		Sets the number of elements of the antenna array. \n
			:return: nx: No help available
		"""
		response = self._core.io.query_str('ANTenna:MODel:ARRay:NX?')
		return Conversions.str_to_float(response)

	def set_nx(self, nx: float) -> None:
		"""SCPI: ANTenna:MODel:ARRay:NX \n
		Snippet: driver.antenna.model.array.set_nx(nx = 1.0) \n
		Sets the number of elements of the antenna array. \n
			:param nx: float Range: 2 to 1000 (planar phased array; linear phase array) , 100 (rectangular phase array) , 50 (hexagonal phase array)
		"""
		param = Conversions.decimal_value_to_str(nx)
		self._core.io.write(f'ANTenna:MODel:ARRay:NX {param}')

	def get_nz(self) -> float:
		"""SCPI: ANTenna:MODel:ARRay:NZ \n
		Snippet: value: float = driver.antenna.model.array.get_nz() \n
		Sets the number of elements of the antenna array. \n
			:return: nz: float Range: 2 to 1000 (planar phased array; linear phase array) , 100 (rectangular phase array) , 50 (hexagonal phase array)
		"""
		response = self._core.io.query_str('ANTenna:MODel:ARRay:NZ?')
		return Conversions.str_to_float(response)

	def set_nz(self, nz: float) -> None:
		"""SCPI: ANTenna:MODel:ARRay:NZ \n
		Snippet: driver.antenna.model.array.set_nz(nz = 1.0) \n
		Sets the number of elements of the antenna array. \n
			:param nz: float Range: 2 to 1000 (planar phased array; linear phase array) , 100 (rectangular phase array) , 50 (hexagonal phase array)
		"""
		param = Conversions.decimal_value_to_str(nz)
		self._core.io.write(f'ANTenna:MODel:ARRay:NZ {param}')

	def get_pedestal(self) -> float:
		"""SCPI: ANTenna:MODel:ARRay:PEDestal \n
		Snippet: value: float = driver.antenna.model.array.get_pedestal() \n
		Sets the Pedestal of the Planar Phased Array antenna. \n
			:return: pedestal: float Range: 0 to 1
		"""
		response = self._core.io.query_str('ANTenna:MODel:ARRay:PEDestal?')
		return Conversions.str_to_float(response)

	def set_pedestal(self, pedestal: float) -> None:
		"""SCPI: ANTenna:MODel:ARRay:PEDestal \n
		Snippet: driver.antenna.model.array.set_pedestal(pedestal = 1.0) \n
		Sets the Pedestal of the Planar Phased Array antenna. \n
			:param pedestal: float Range: 0 to 1
		"""
		param = Conversions.decimal_value_to_str(pedestal)
		self._core.io.write(f'ANTenna:MODel:ARRay:PEDestal {param}')

	def get_xdistance(self) -> float:
		"""SCPI: ANTenna:MODel:ARRay:XDIStance \n
		Snippet: value: float = driver.antenna.model.array.get_xdistance() \n
		Sets the spacing between the elements of the array antenna. \n
			:return: xdistance: No help available
		"""
		response = self._core.io.query_str('ANTenna:MODel:ARRay:XDIStance?')
		return Conversions.str_to_float(response)

	def set_xdistance(self, xdistance: float) -> None:
		"""SCPI: ANTenna:MODel:ARRay:XDIStance \n
		Snippet: driver.antenna.model.array.set_xdistance(xdistance = 1.0) \n
		Sets the spacing between the elements of the array antenna. \n
			:param xdistance: float Range: 0.0001 to 1
		"""
		param = Conversions.decimal_value_to_str(xdistance)
		self._core.io.write(f'ANTenna:MODel:ARRay:XDIStance {param}')

	def get_zdistance(self) -> float:
		"""SCPI: ANTenna:MODel:ARRay:ZDIStance \n
		Snippet: value: float = driver.antenna.model.array.get_zdistance() \n
		Sets the spacing between the elements of the array antenna. \n
			:return: zdistance: float Range: 0.0001 to 1
		"""
		response = self._core.io.query_str('ANTenna:MODel:ARRay:ZDIStance?')
		return Conversions.str_to_float(response)

	def set_zdistance(self, zdistance: float) -> None:
		"""SCPI: ANTenna:MODel:ARRay:ZDIStance \n
		Snippet: driver.antenna.model.array.set_zdistance(zdistance = 1.0) \n
		Sets the spacing between the elements of the array antenna. \n
			:param zdistance: float Range: 0.0001 to 1
		"""
		param = Conversions.decimal_value_to_str(zdistance)
		self._core.io.write(f'ANTenna:MODel:ARRay:ZDIStance {param}')

	def clone(self) -> 'Array':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = Array(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group
